'
' MediaMonkey Script
'
' NAME: AdvancedReport 6.4
'
' AUTHOR: trixmoto (http://trixmoto.net)
' DATE : 24/06/2014
'
' INSTALL: Copy to Scripts directory and add the following to Scripts.ini 
'          Don't forget to remove comments (') and set the order appropriately
'
' Thanks to: ZvezdanD for his modifications to Magic Nodes which ignore prefixes
'            Psyxonova for his expert SQL help
'            Buxfix for making the output XHTML and providing new stylesheet
'            Bex for his help with SQLite optimisation and filters and selected tracks
'            Tralla for her modified versions which I've taken many ideas from
'            Jn and Gege for letting me use their WebSearch Panels script
'
' [AdvancedReport]
' FileName=AdvancedReport.vbs
' ProcName=AdvancedReport
' Order=22
' DisplayName=Advanced Report
' Description=Advanced Report
' Language=VBScript
' ScriptType=1
'
' Album mask fields:
' 	<Name><Artist><Length><Tracks><Rating><Stars><Year><Bombs><Discs><Comment><Genre>
'
' Track mask fields:
' 	<Track#><Artist><Title><Album><Album Artist><Genre><Year><Bitrate><BPM>
'   <Composer><Custom 1><Custom 2><Custom 3><Filename><Length><Rating><Stars>
'   <Bombs><|><Band><Channels><MusicComposer><Conductor><Copyright><Custom 4><Custom 5>
'   <Disc#><Grouping><Lyricist><Mood><Occasion><Publisher><Quality><Tempo><Encoder>
'   <Link><ArtistLink><TitleLink><Comment><MediaIcon><Date><OrigYear><Lyrics>
'   <StartTime><StopTime><SkipCount><TrackType>
'   <Series><Director><Producer><Actors><ParentalRating><EpisodeNumber><SeasonNumber>
'
' Tooltip mask fields:
' 	<Track#><Artist><Title><Album><Album Artist><Genre><Year><Bitrate><BPM>
'   <Composer><Custom 1><Custom 2><Custom 3><Filename><Length><Rating>
'   <Band><Channels><MusicComposer><Conductor><Copyright><Custom 4><Custom 5>
'   <Disc#><Grouping><Lyricist><Mood><Occasion><Publisher><Quality><Tempo><Encoder>
'   <Comment><Date><OrigYear><Lyrics><StartTime><StopTime><SkipCount><TrackType>
'   <Series><Director><Producer><Actors><ParentalRating><EpisodeNumber><SeasonNumber>
'
' Artist mask fields:
'   <Name><Tracks><Albums><Authors><Conducts><Lyrics><Rating><Year><Stars><Bombs><Comment>
'
' FIXES: Made output HTML5 compliant and updated DOCTYPE
'        Fixed errors if prefixes contain an apostrophe
'

Option Explicit 
Dim AppName : AppName = "Advanced Report 6.4"
Dim Debug : Debug = False
Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
Dim Progress,Artwork,Path,UnknownArtist,UnknownAlbum,SeparateFiles
Dim TrackMask,AlbumMask,ScreenWidth,AlbumArtWidth,AlbumArtIcon,AlbumArtMode
Dim GroupNonAlpha,AlbumArtists,IgnoreThes,PageTitle,ListGenres,ListYears
Dim SupportUnicode,ApplyMask,TrackLinks,FramedView,AlbumsByYear,NewImages
Dim IncludeMode,ShowUnknown,CompAlbumMask,CompTrackMask,AlbumsPerRow,Playlists
Dim Filters(),FilterName,FilterSQL,FromClause1,FromClause2,WhereClause1,WhereClause2
Dim BaseDir,ImgsDir,ArtsDir,AlbsDir,StylDir,PlayDir,FramedMode,DisplayTop,TooltipMask
Dim TableHeader,ImageBase,ArtistMask,ListArtGenres,ListArtYears,Delimiter,ShowIcons
Dim ShowLinks,ShowTracks,ShowStats,DetailsMask,RecentLimit,PlayedLimit,RatedLimit

Sub AdvancedReport
  'default values
  TrackMask = "<Track#><|><Title><|><Artist><|><Stars>"
  AlbumMask = "<Name> (<Artist>) [<Year>] - <Rating>/5"
  AlbumArtMode = 0
  AlbumArtWidth = 200
  AlbumArtIcon = 100
  ScreenWidth = 1024
  SeparateFiles = False
  UnknownArtist = "(Unknown Artist)"
  UnknownAlbum = "(Unknown Album)"
  Path = "C:\Report\index.htm"
  GroupNonAlpha = False
  AlbumArtists = False
  IgnoreThes = False
  SupportUnicode = False
  ApplyMask = False
  TrackLinks = 0
  FramedView = False
  AlbumsByYear = False
  NewImages = False
  PageTitle = "MediaMonkey "&SDB.VersionHi&"."&SDB.VersionLo&" Track List"
  ListGenres = False
  FilterName = "- No Filter -"
  IncludeMode = 0
  ShowUnknown = False
  CompAlbumMask = ""
  CompTrackMask = ""
  AlbumsPerRow = 0
  Playlists = False
  FramedMode = 0
  DisplayTop = 0
  ListYears = False
  TooltipMask = ""
  TableHeader = False
  ArtistMask = "<Name>"
  ListArtGenres = False
  ListArtYears = False
  ShowIcons = "A0;B0;C0;D0;E0;F0;G0;H0;I0;J0;K0;L0;M0;N0;O0;P0;Q0;R0;S0;T0;U0;V0;W0;X0;Y0;Z0"
  ShowLinks = False
  ShowTracks = True
  ShowStats = False
  DetailsMask = ""
  RecentLimit = 5
  PlayedLimit = 5
  RatedLimit = 5
  
  'read ini file
  Dim ini : Set ini = SDB.IniFile
  Dim temp : temp = ini.StringValue("AdvancedReport","Path")
  If Not temp = "" Then 
    Path = temp
  End If
  temp = ini.StringValue("AdvancedReport","TrackMask")
  If Not temp = "" Then 
    TrackMask = temp
  End If
  temp = ini.StringValue("AdvancedReport","AlbumMask")
  If Not temp = "" Then 
    AlbumMask = temp
  End If
  temp = ini.StringValue("AdvancedReport","AlbumArtMode")
  If IsNumeric(temp) Then 
    AlbumArtMode = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","AlbumArtWidth")
  If IsNumeric(temp) Then 
    AlbumArtWidth = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","AlbumArtIcon")
  If IsNumeric(temp) Then 
    AlbumArtIcon = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","ScreenWidth")
  If IsNumeric(temp) Then 
    ScreenWidth = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","SeparateFiles")
  If temp = "1" Then 
    SeparateFiles = True
  End If
  If temp = "2" Then 
    FramedView = True
  End If
  FramedMode = 0
  If temp = "3" Then
    FramedView = True
    FramedMode = 1
  End If
  temp = ini.StringValue("AdvancedReport","UnknownArtist")
  If Not temp = "" Then 
    UnknownArtist = temp
  End If
  temp = ini.StringValue("AdvancedReport","UnknownAlbum")
  If Not temp = "" Then 
    UnknownAlbum = temp
  End If
  temp = ini.StringValue("AdvancedReport","GroupNonAlpha")
  If temp = "1" Then 
    GroupNonAlpha = True
  End If
  temp = ini.StringValue("AdvancedReport","AlbumArtists")
  If temp = "1" Then 
    AlbumArtists = True
  End If  
  temp = ini.StringValue("AdvancedReport","IgnoreThes")
  If temp = "1" Then 
    IgnoreThes = True
  End If  
  temp = ini.StringValue("AdvancedReport","SupportUnicode")
  If temp = "1" Then 
    SupportUnicode = True
  End If
  temp = ini.StringValue("AdvancedReport","ApplyMask")
  If temp = "1" Then 
    ApplyMask = True
  End If
  temp = ini.StringValue("AdvancedReport","TrackLinks")
  If Not temp = "" Then 
    TrackLinks = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","AlbumsByYear")
  If temp = "1" Then 
    AlbumsByYear = True
  End If  
  temp = ini.StringValue("AdvancedReport","NewImages")
  If temp = "1" Then 
    NewImages = True
  End If   
  temp = ini.StringValue("AdvancedReport","PageTitle")
  If Not temp = "" Then 
    PageTitle = temp
  End If
  temp = ini.StringValue("AdvancedReport","ListGenres")
  If temp = "1" Then 
    ListGenres = True
  End If   
  temp = ini.StringValue("AdvancedReport","FilterName")
  If Not (temp = "") Then 
    FilterName = temp
  End If
  temp = ini.StringValue("AdvancedReport","IncludeMode")
  If IsNumeric(temp) Then 
    IncludeMode = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","ShowUnknown")
  If temp = "1" Then 
    ShowUnknown = True
  End If 
  temp = ini.StringValue("AdvancedReport","CompAlbumMask")
  If Not temp = "" Then 
    CompAlbumMask = temp
  End If
  temp = ini.StringValue("AdvancedReport","CompTrackMask")
  If Not temp = "" Then
    CompTrackMask = temp
  End If
  temp = ini.StringValue("AdvancedReport","AlbumsPerRow")
  If Not temp = "" Then 
    AlbumsPerRow = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","Playlists")
  If temp = "1" Then 
    Playlists = True
  End If 
  temp = ini.StringValue("AdvancedReport","DisplayTop")
  If Not temp = "" Then
    DisplayTop = temp
  End If
  temp = ini.StringValue("AdvancedReport","ListYears")
  If temp = "1" Then 
    ListYears = True
  End If   
  temp = ini.StringValue("AdvancedReport","TooltipMask")
  If Not temp = "" Then 
    TooltipMask = temp
  End If
  temp = ini.StringValue("AdvancedReport","TableHeader")
  If temp = "1" Then 
    TableHeader = True
  End If
  temp = ini.StringValue("AdvancedReport","ArtistMask")
  If Not temp = "" Then 
    ArtistMask = temp
  End If
  temp = ini.StringValue("AdvancedReport","ListArtGenres")
  If temp = "1" Then 
    ListArtGenres = True
  End If
  temp = ini.StringValue("AdvancedReport","ListArtYears")
  If temp = "1" Then 
    ListArtYears = True
  End If               
  temp = ini.StringValue("AdvancedReport","ShowIcons")
  If Not temp = "" Then 
    ShowIcons = temp
  End If
  temp = ini.StringValue("AdvancedReport","ShowLinks")
  If temp = "1" Then 
    ShowLinks = True
  End If
  temp = ini.StringValue("AdvancedReport","ShowTracks")
  If temp = "0" Then 
    ShowTracks = False
  End If
  temp = ini.StringValue("AdvancedReport","ShowStats")
  If temp = "1" Then 
    ShowStats = True
  End If
  temp = ini.StringValue("AdvancedReport","DetailsMask")
  If Not temp = "" Then 
    DetailsMask = temp
  End If              
  temp = ini.StringValue("AdvancedReport","RecentLimit")
  If Not temp = "" Then 
    RecentLimit = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","PlayedLimit")
  If Not temp = "" Then 
    PlayedLimit = Int(temp)
  End If
  temp = ini.StringValue("AdvancedReport","RatedLimit")
  If Not temp = "" Then 
    RatedLimit = Int(temp)
  End If      
  
  'create form
  Dim lblLeft : lblLeft = 10
  Dim edtLeft : edtLeft = 100

  Dim Form : Set Form = SDB.UI.NewForm
  Form.Common.SetRect 100, 100, 612, 620
  Form.BorderStyle  = 2   
  Form.FormPosition = 4   
  Form.Caption = AppName&" - Options"

  Dim Label1 : Set Label1 = SDB.UI.NewLabel(Form)
  Label1.Caption = "Output path"
  Label1.Common.Left = lblLeft
  Label1.Common.Top = 15
         
  Dim Edt1 : Set Edt1 = SDB.UI.NewEdit(Form)
  Edt1.Common.Left = edtLeft
  Edt1.Common.Top = Label1.Common.Top -4
  Edt1.Common.Width = Form.Common.Width - Edt1.Common.Left -25
  Edt1.Common.ControlName = "Path"
  Edt1.Text = Path
  
  Dim Browse : Set Browse = SDB.UI.NewButton(Form)
  Browse.Caption = "..."
  Browse.Common.Height = Edt1.Common.Height
  Browse.Common.Width = Edt1.Common.Height
  Browse.Common.Top = Edt1.Common.Top
  Browse.Common.Left = Edt1.Common.Left - Browse.Common.Width -5
  Browse.UseScript = Script.ScriptPath
  Browse.OnClickFunc = "BrowseClick"
  
  Dim Label2 : Set Label2 = SDB.UI.NewLabel(Form)
  Label2.Caption = "Track mask"
  Label2.Common.Left = lblLeft
  Label2.Common.Top = Label1.Common.Top +25
    
  Dim Edt2 : Set Edt2 = SDB.UI.NewEdit(Form)
  Edt2.Common.Left = edtLeft
  Edt2.Common.Top = Label2.Common.Top -4
  Edt2.Common.Width = Form.Common.Width - Edt2.Common.Left -25
  Edt2.Common.Hint = "<Track#><Artist><Title><Album><Album Artist><Genre><Year><Bitrate><BPM>"&chr(13)&"<Composer><Custom 1><Custom 2><Custom 3><Filename><Length><Rating><Stars>"&chr(13)&"<Bombs><Band><Channels><MusicComposer><Conductor><Copyright><Custom 4><Custom 5>"&chr(13)&"<Disc#><Grouping><Lyricist><Mood><Occasion><Publisher><Quality><Tempo><Encoder>"&chr(13)&"<Link><ArtistLink><TitleLink><Comment><MediaIcon><Date><OrigYear><Lyrics><StartTime><StopTime><SkipCount><TrackType>"&chr(13)&"<Series><Director><Producer><Actors><ParentalRating><EpisodeNumber><SeasonNumber>"
  Edt2.Text = TrackMask
  
  Dim Label3 : Set Label3 = SDB.UI.NewLabel(Form)
  Label3.Caption = "Album mask"
  Label3.Common.Left = lblLeft
  Label3.Common.Top = Label2.Common.Top +25
    
  Dim Edt3 : Set Edt3 = SDB.UI.NewEdit(Form)
  Edt3.Common.Left = edtLeft
  Edt3.Common.Top = Label3.Common.Top -4
  Edt3.Common.Width = Form.Common.Width - Edt3.Common.Left -25
  Edt3.Common.Hint = "<Name><Artist><Length><Tracks><Rating><Stars><Year><Bombs><Discs><Comment><Genre>"
  Edt3.Text = AlbumMask
  
  Dim Label30 : Set Label30 = SDB.UI.NewLabel(Form)
  Label30.Caption = "Comp track mask"
  Label30.Common.Left = lblLeft
  Label30.Common.Top = Label3.Common.Top +25
  
  Dim Edt30 : Set Edt30 = SDB.UI.NewEdit(Form)
  Edt30.Common.Left = edtLeft
  Edt30.Common.Top = Label30.Common.Top -4
  Edt30.Common.Width = Form.Common.Width - Edt30.Common.Left -25
  Edt30.Common.Hint = Edt2.Common.Hint
  Edt30.Text = CompTrackMask
  
  Dim Label31 : Set Label31 = SDB.UI.NewLabel(Form)
  Label31.Caption = "Comp album mask"
  Label31.Common.Left = lblLeft
  Label31.Common.Top = Label30.Common.Top +25
  
  Dim Edt31 : Set Edt31 = SDB.UI.NewEdit(Form)
  Edt31.Common.Left = edtLeft
  Edt31.Common.Top = Label31.Common.Top -4
  Edt31.Common.Width = Form.Common.Width - Edt31.Common.Left -25
  Edt31.Common.Hint = Edt3.Common.Hint
  Edt31.Text = CompAlbumMask
  
  Dim Label37 : Set Label37 = SDB.UI.NewLabel(Form)
  Label37.Caption = "Tooltip mask"
  Label37.Common.Left = lblLeft
  Label37.Common.Top = Label31.Common.Top +25
    
  Dim Edt37 : Set Edt37 = SDB.UI.NewEdit(Form)
  Edt37.Common.Left = edtLeft
  Edt37.Common.Top = Label37.Common.Top -4
  Edt37.Common.Width = Form.Common.Width - Edt37.Common.Left -25
  Edt37.Common.Hint = "<Track#><Artist><Title><Album><Album Artist><Genre><Year><Bitrate><BPM>"&chr(13)&"<Composer><Custom 1><Custom 2><Custom 3><Filename><Length><Rating>"&chr(13)&"<Band><Channels><MusicComposer><Conductor><Copyright><Custom 4><Custom 5>"&chr(13)&"<Disc#><Grouping><Lyricist><Mood><Occasion><Publisher><Quality><Tempo><Encoder><Comment><Date><OrigYear><Lyrics><StartTime><StopTime><SkipCount><TrackType>"&chr(13)&"<Series><Director><Producer><Actors><ParentalRating><EpisodeNumber><SeasonNumber>"
  Edt37.Text = TooltipMask
  
  Dim Label39 : Set Label39 = SDB.UI.NewLabel(Form)
  Label39.Caption = "Artist mask"
  Label39.Common.Left = lblLeft
  Label39.Common.Top = Label37.Common.Top +25
    
  Dim Edt39 : Set Edt39 = SDB.UI.NewEdit(Form)
  Edt39.Common.Left = edtLeft
  Edt39.Common.Top = Label39.Common.Top -4
  Edt39.Common.Width = Form.Common.Width - Edt39.Common.Left -25
  Edt39.Common.Hint = "<Name><Tracks><Albums><Authors><Conducts><Lyrics><Rating><Year><Stars><Bombs><Comment>"
  Edt39.Text = ArtistMask
  
  Dim Label45 : Set Label45 = SDB.UI.NewLabel(Form)
  Label45.Caption = "Album info mask"
  Label45.Common.Left = lblLeft
  Label45.Common.Top = Label39.Common.Top +25
  
  Dim Edt45 : Set Edt45 = SDB.UI.NewEdit(Form)
  Edt45.Common.Left = edtLeft
  Edt45.Common.Top = Label45.Common.Top -4
  Edt45.Common.Width = Form.Common.Width - Edt45.Common.Left -25
  Edt45.Common.Hint = Edt3.Common.Hint
  Edt45.Text = DetailsMask   
  
  Dim Label4 : Set Label4 = SDB.UI.NewLabel(Form)
  Label4.Caption = "Album art mode"
  Label4.Common.Left = lblLeft
  Label4.Common.Top = Label45.Common.Top +25
  
  Dim Edt4 : Set Edt4 = SDB.UI.NewDropDown(Form)
  Edt4.Common.Left = edtLeft
  Edt4.Common.Top = Label4.Common.Top -4
  Edt4.Common.Width = 100
  Edt4.AddItem("Fixed width")
  Edt4.AddItem("Fixed height")
  Edt4.AddItem("Fixed both")
  Edt4.AddItem("Do not display")
  Edt4.Style = 2
  Edt4.ItemIndex = AlbumArtMode
  
  Dim Label5 : Set Label5 = SDB.UI.NewLabel(Form)
  Label5.Caption = "Width"
  Label5.Common.Left = Edt4.Common.Left + Edt4.Common.Width +10
  Label5.Common.Top = Label4.Common.Top
      
  Dim Edt5 : Set Edt5 = SDB.UI.NewEdit(Form)
  Edt5.Common.Left = Label5.Common.Left + Label5.Common.Width +10
  Edt5.Common.Top = Label5.Common.Top -4
  Edt5.Common.Width = 43
  Edt5.Text = AlbumArtWidth
  
  Dim Label6 : Set Label6 = SDB.UI.NewLabel(Form)
  Label6.Caption = "Icon"
  Label6.Common.Left = Edt5.Common.Left + Edt5.Common.Width +10
  Label6.Common.Top = Label5.Common.Top
      
  Dim Edt6 : Set Edt6 = SDB.UI.NewEdit(Form)
  Edt6.Common.Left = Label6.Common.Left + Label6.Common.Width +10
  Edt6.Common.Top = Label6.Common.Top -4
  Edt6.Common.Width = 43
  Edt6.Text = AlbumArtIcon
  
  Dim Label13 : Set Label13 = SDB.UI.NewLabel(Form)
  Label13.Caption = "Screen width"
  Label13.Common.Left = lblLeft
  Label13.Common.Top = Label5.Common.Top +25
      
  Dim Edt13 : Set Edt13 = SDB.UI.NewEdit(Form)
  Edt13.Common.Left = edtLeft
  Edt13.Common.Top = Label13.Common.Top -4
  Edt13.Common.Width = 100
  Edt13.Text = ScreenWidth
      
  Dim Label14 : Set Label14 = SDB.UI.NewLabel(Form)
  Label14.Caption = "Style"
  Label14.Common.Left = Label5.Common.Left
  Label14.Common.Top = Label13.Common.Top
      
  Dim Edt14 : Set Edt14 = SDB.UI.NewDropDown(Form)
  Edt14.Common.Left = Edt5.Common.Left
  Edt14.Common.Top = Label14.Common.Top -4
  Edt14.Common.Width = (Edt6.Common.Left + Edt6.Common.Width) - Edt5.Common.Left
  Edt14.AddItem("Single file")
  Edt14.AddItem("Separate files")
  Edt14.AddItem("Frame set left")
  Edt14.AddItem("Frame set top")
  Edt14.Style = 2
  If SeparateFiles Then
    Edt14.ItemIndex = 1
  ElseIf FramedView Then
    If FramedMode = 0 Then
      Edt14.ItemIndex = 2
    Else
      Edt14.ItemIndex = 3
    End If
  Else
    Edt14.ItemIndex = 0
  End If
  
  Dim Label15 : Set Label15 = SDB.UI.NewLabel(Form)
  Label15.Caption = "Unknown artist"
  Label15.Common.Left = lblLeft
  Label15.Common.Top = Label13.Common.Top +25
    
  Dim Edt15 : Set Edt15 = SDB.UI.NewEdit(Form)
  Edt15.Common.Left = edtLeft
  Edt15.Common.Top = Label15.Common.Top -4
  Edt15.Common.Width = Form.Common.Width - Edt15.Common.Left -25
  Edt15.Text = UnknownArtist
  
  Dim Label16 : Set Label16 = SDB.UI.NewLabel(Form)
  Label16.Caption = "Unknown album"
  Label16.Common.Left = lblLeft
  Label16.Common.Top = Label15.Common.Top +25
  
  Dim Edt16 : Set Edt16 = SDB.UI.NewEdit(Form)
  Edt16.Common.Left = edtLeft
  Edt16.Common.Top = Label16.Common.Top -4
  Edt16.Common.Width = Form.Common.Width - Edt16.Common.Left -25
  Edt16.Text = UnknownAlbum
  
  Dim Label25 : Set Label25 = SDB.UI.NewLabel(Form)
  Label25.Caption = "Page title"
  Label25.Common.Left = lblLeft
  Label25.Common.Top = Label16.Common.Top +25  
  
  Dim Edt25 : Set Edt25 = SDB.UI.NewEdit(Form)
  Edt25.Common.Left = edtLeft
  Edt25.Common.Top = Label25.Common.Top -4
  Edt25.Common.Width = Form.Common.Width - Edt25.Common.Left -25
  Edt25.Text = PageTitle
  
  Dim Label28 : Set Label28 = SDB.UI.NewLabel(Form)
  Label28.Caption = "Include albums"
  Label28.Common.Left = lblLeft
  Label28.Common.Top = Label25.Common.Top +25  
  
  Dim Edt28 : Set Edt28 = SDB.UI.NewDropDown(Form)
  Edt28.Common.Left = edtLeft
  Edt28.Common.Top = Label28.Common.Top -4
  Edt28.Common.Width = 100
  Edt28.AddItem("All (mixed)")
  Edt28.AddItem("By artist")
  Edt28.AddItem("Featuring")
  Edt28.AddItem("All (split)")
  Edt28.Style = 2
  Edt28.ItemIndex = IncludeMode

  Dim Edt27,Label27  
  Set Label27 = SDB.UI.NewLabel(Form)
  Label27.Caption = "Filter"
  Label27.Common.Left = Label5.Common.Left
  Label27.Common.Top = Label28.Common.Top    
    
  Set Edt27 = SDB.UI.NewDropDown(Form)
  Edt27.Common.Left = Edt5.Common.Left
  Edt27.Common.Top = Edt28.Common.Top
  Edt27.Common.Width = Form.Common.Width - Edt27.Common.Left -25
  Edt27.Style = 2
  Edt27.Common.ControlName = "drdFilters"
  Call CreateFiltersArray()
  Call FillDropDownFromArray(Edt27,Filters)
  Edt27.ItemIndex = GetFiltersArrayID(FilterName)
  
  Dim Label32 : Set Label32 = SDB.UI.NewLabel(Form)
  Label32.Caption = "Albums per row"
  Label32.Common.Left = lblLeft
  Label32.Common.Top = Label28.Common.Top +25
  
  Dim Edt32 : Set Edt32 = SDB.UI.NewSpinEdit(Form)
  Edt32.Common.Left = edtLeft
  Edt32.Common.Top = Label32.Common.Top -4
  Edt32.Common.Width = 100  
  Edt32.Common.Hint = "0 = Automatic"
  Edt32.MinValue = 0
  Edt32.MaxValue = 99
  Edt32.Value = AlbumsPerRow
  
  Dim Edt17 : Set Edt17 = SDB.UI.NewCheckbox(Form)
  Edt17.Caption = "Group all non-alpha characters"
  Edt17.Common.Left = lblLeft
  Edt17.Common.Top = Label32.Common.Top +25
  Edt17.Common.Width = 300
  Edt17.Checked = GroupNonAlpha

  Dim Edt18 : Set Edt18 = SDB.UI.NewCheckbox(Form)
  Edt18.Caption = "Only include album artists"
  Edt18.Common.Left = lblLeft
  Edt18.Common.Top = Edt17.Common.Top +25
  Edt18.Common.Width = 300
  Edt18.Checked = AlbumArtists
  
  Dim thes : thes = ini.StringValue("Options","IgnoreTHEStrings")
  If (thes = "") Or Not (ini.BoolValue("Options","IgnoreTHEs")) Then
    thes = ""
    IgnoreThes = False
  Else
    thes = " ("&thes&")"
  End If  
  
  Dim Edt19 : Set Edt19 = SDB.UI.NewCheckbox(Form)
  Edt19.Caption = "Ignore prefixes"&thes
  Edt19.Common.Left = lblLeft
  Edt19.Common.Top = Edt18.Common.Top +25
  Edt19.Common.Width = 300
  Edt19.Checked = IgnoreThes
  If thes = "" Then
    Edt19.Common.Enabled = False
  End If
  
  Dim Edt20 : Set Edt20 = SDB.UI.NewCheckbox(Form)
  Edt20.Caption = "Support unicode characters"
  Edt20.Common.Left = lblLeft
  Edt20.Common.Top = Edt19.Common.Top +25
  Edt20.Common.Width = 300
  Edt20.Checked = SupportUnicode
  
  Dim Edt33 : Set Edt33 = SDB.UI.NewCheckbox(Form)
  Edt33.Caption = "Include playlists page"
  Edt33.Common.Left = Label5.Common.Left
  Edt33.Common.Top = Edt19.Common.Top
  Edt33.Common.Width = 200
  Edt33.Checked = Playlists  
  
  Dim Edt21 : Set Edt21 = SDB.UI.NewCheckbox(Form)
  Edt21.Caption = "Apply album mask to artist list"
  Edt21.Common.Left = Label5.Common.Left
  Edt21.Common.Top = Edt17.Common.Top
  Edt21.Common.Width = 200
  Edt21.Checked = ApplyMask   
  
  Dim Edt23 : Set Edt23 = SDB.UI.NewCheckbox(Form)
  Edt23.Caption = "Sort albums by year"
  Edt23.Common.Left = Label5.Common.Left
  Edt23.Common.Top = Edt18.Common.Top
  Edt23.Common.Width = 200
  Edt23.Checked = AlbumsByYear  
  
  Dim Edt43 : Set Edt43 = SDB.UI.NewCheckbox(Form)
  Edt43.Caption = "Include track listing"
  Edt43.Common.Left = Label5.Common.Left
  Edt43.Common.Top = Edt32.Common.Top +3
  Edt43.Common.Width = 150
  Edt43.Checked = ShowTracks
  
  Dim Label26 : Set Label26 = SDB.UI.NewLabel(Form)
  Label26.Caption = "List album:"
  Label26.Common.Left = Edt20.Common.Left
  Label26.Common.Top = Edt20.Common.Top +28  
  
  Dim Edt26 : Set Edt26 = SDB.UI.NewCheckbox(Form)
  Edt26.Caption = "genres"
  Edt26.Common.Left = lblLeft+60
  Edt26.Common.Top = Label26.Common.Top -3
  Edt26.Common.Width = 100
  Edt26.Checked = ListGenres
  
  Dim Edt36 : Set Edt36 = SDB.UI.NewCheckbox(Form)
  Edt36.Caption = "years"
  Edt36.Common.Left = lblLeft+120
  Edt36.Common.Top = Edt26.Common.Top
  Edt36.Common.Width = 50
  Edt36.Checked = ListYears  
  
  Dim Edt29 : Set Edt29 = SDB.UI.NewCheckbox(Form)
  Edt29.Caption = "Include unknown album"
  Edt29.Common.Left = Label5.Common.Left
  Edt29.Common.Top = Edt20.Common.Top
  Edt29.Common.Width = 200
  Edt29.Checked = ShowUnknown
  
  Dim Label40 : Set Label40 = SDB.UI.NewLabel(Form)
  Label40.Caption = "List artist:"
  Label40.Common.Left = Label26.Common.Left
  Label40.Common.Top = Label26.Common.Top +25  
  
  Dim Edt40 : Set Edt40 = SDB.UI.NewCheckbox(Form)
  Edt40.Caption = "genres"
  Edt40.Common.Left = Label40.Common.Left +60
  Edt40.Common.Top = Label40.Common.Top -4
  Edt40.Common.Width = 100
  Edt40.Checked = ListArtGenres
  
  Dim Edt41 : Set Edt41 = SDB.UI.NewCheckbox(Form)
  Edt41.Caption = "years"
  Edt41.Common.Left = Label40.Common.Left +120
  Edt41.Common.Top = Edt40.Common.Top
  Edt41.Common.Width = 50
  Edt41.Checked = ListArtYears    
  
  Dim Label34 : Set Label34 = SDB.UI.NewLabel(Form)
  Label34.Caption = "Description"
  Label34.Common.Left = 390
  Label34.Common.Top = Label5.Common.Top
      
  Dim Edt34 : Set Edt34 = SDB.UI.NewDropDown(Form)
  Edt34.Common.Left = Label34.Common.Left +70
  Edt34.Common.Top = Label34.Common.Top -4
  Edt34.Common.Width = Edt14.Common.Width
  Edt34.AddItem("Above image")
  Edt34.AddItem("Below image")
  Edt34.Style = 2
  Edt34.ItemIndex = DisplayTop
  
  Dim Label35 : Set Label35 = SDB.UI.NewLabel(Form)
  Label35.Caption = "Track links"
  Label35.Common.Left = 390
  Label35.Common.Top = Label34.Common.Top +25
  
  Dim Edt35 : Set Edt35 = SDB.UI.NewDropDown(Form)
  Edt35.Common.Left = Label35.Common.Left +70
  Edt35.Common.Top = Label35.Common.Top -4
  Edt35.Common.Width = Edt34.Common.Width
  Edt35.AddItem("None")
  Edt35.AddItem("Absolute")
  Edt35.AddItem("Relative")
  Edt35.Style = 2
  Edt35.ItemIndex = TrackLinks
  
  Dim Edt38 : Set Edt38 = SDB.UI.NewCheckbox(Form)
  Edt38.Caption = "Include table header in track listing"
  Edt38.Common.Left = Label35.Common.Left
  Edt38.Common.Top = Edt43.Common.Top
  Edt38.Common.Width = 200
  Edt38.Checked = TableHeader  
  
  Dim Edt42 : Set Edt42 = SDB.UI.NewCheckbox(Form)
  Edt42.Caption = "Include 'WebSearch' links"
  Edt42.Common.Left = Edt29.Common.Left
  Edt42.Common.Top = Edt29.Common.Top +25
  Edt42.Common.Width = 200
  If Not fso.FileExists(SDB.ScriptsPath&"\Auto\WebSearch.xml") Then
    Edt42.Common.Enabled = False
    ShowLinks = False
  End If            
  Edt42.Checked = ShowLinks
  
  Dim Edt24 : Set Edt24 = SDB.UI.NewCheckbox(Form)
  Edt24.Caption = "Only create new images"
  Edt24.Common.Left = Edt42.Common.Left
  Edt24.Common.Top = Edt42.Common.Top +25
  Edt24.Common.Width = 200
  Edt24.Checked = NewImages  
  
  Dim Edt44 : Set Edt44 = SDB.UI.NewCheckbox(Form)
  Edt44.Caption = "Include top albums page, with..."
  Edt44.Common.Left = Edt38.Common.Left
  Edt44.Common.Top = Edt21.Common.Top
  Edt44.Common.Width = 200
  Edt44.Checked = ShowStats
  
  Dim Label46 : Set Label46 = SDB.UI.NewLabel(Form)
  Label46.Caption = "Recently added"
  Label46.Common.Left = Edt44.Common.Left +50
  Label46.Common.Top = Edt44.Common.Top +25
  
  Dim Edt46 : Set Edt46 = SDB.UI.NewSpinEdit(Form)
  Edt46.Common.Left = Form.Common.Width - 75
  Edt46.Common.Top = Label46.Common.Top -4
  Edt46.Common.Width = 50  
  Edt46.MinValue = 0
  Edt46.MaxValue = 99
  Edt46.Value = RecentLimit
  
  Dim Label47 : Set Label47 = SDB.UI.NewLabel(Form)
  Label47.Caption = "Top played"
  Label47.Common.Left = Label46.Common.Left
  Label47.Common.Top = Label46.Common.Top +25
  
  Dim Edt47 : Set Edt47 = SDB.UI.NewSpinEdit(Form)
  Edt47.Common.Left = Form.Common.Width - 75
  Edt47.Common.Top = Label47.Common.Top -4
  Edt47.Common.Width = 50  
  Edt47.MinValue = 0
  Edt47.MaxValue = 99
  Edt47.Value = PlayedLimit     
    
  Dim Label48 : Set Label48 = SDB.UI.NewLabel(Form)
  Label48.Caption = "Top rated"
  Label48.Common.Left = Label46.Common.Left
  Label48.Common.Top = Label47.Common.Top +25
  
  Dim Edt48 : Set Edt48 = SDB.UI.NewSpinEdit(Form)
  Edt48.Common.Left = Form.Common.Width - 75
  Edt48.Common.Top = Label48.Common.Top -4
  Edt48.Common.Width = 50  
  Edt48.MinValue = 0
  Edt48.MaxValue = 99
  Edt48.Value = RatedLimit
  
  Dim BtnCancel : Set BtnCancel = SDB.UI.NewButton(Form)
  BtnCancel.Caption = "&Cancel"
  BtnCancel.Common.Top = Form.Common.Height - BtnCancel.Common.Height -35
  BtnCancel.Common.Left = Form.Common.Width - BtnCancel.Common.Width -25
  BtnCancel.UseScript = Script.ScriptPath
  BtnCancel.Cancel = True
  BtnCancel.ModalResult = 2
  
  Dim BtnReport : Set BtnReport = SDB.UI.NewButton(Form)
  BtnReport.Caption = "&Report"
  BtnReport.Common.Top = BtnCancel.Common.Top
  BtnReport.Common.Left = BtnCancel.Common.Left - BtnReport.Common.Width -10
  BtnReport.UseScript = Script.ScriptPath
  BtnReport.Default = True
  BtnReport.ModalResult = 1
  
  Dim BtnEdit : Set BtnEdit = SDB.UI.NewButton(Form)
  BtnEdit.Caption = "&Edit CSS"
  BtnEdit.Common.Top = BtnCancel.Common.Top
  BtnEdit.Common.Left = lblLeft
  BtnEdit.UseScript = Script.ScriptPath
  BtnEdit.OnClickFunc = "EditClick"
  
  Dim BtnIcons : Set BtnIcons = SDB.UI.NewButton(Form)
  BtnIcons.Caption = "Set &Icons"
  BtnIcons.Common.Top = BtnCancel.Common.Top
  BtnIcons.Common.Left = lblLeft + BtnEdit.Common.Width +10
  BtnIcons.UseScript = Script.ScriptPath
  BtnIcons.OnClickFunc = "IconsClick"    
  
  'show form
  If Not (Form.ShowModal = 1) Then 
    Exit Sub
  End If
  
  'initialise script
  Set Progress = SDB.Progress 
  UpdateProgress("Initialising script...")
  Dim starttime : starttime = Timer
  Delimiter = Replace(ini.StringValue("Appearance","MultiStringSeparator"),"""","")
  
  'update settings
  If Not Edt1.Text = "" Then 
    Path = Edt1.Text
  End If
  If Not Edt2.Text = "" Then 
    TrackMask = Edt2.Text
  End If
  If Not Edt3.Text = "" Then 
    AlbumMask = Edt3.Text
  End If
  AlbumArtMode = Edt4.ItemIndex
  If IsNumeric(Edt5.Text) Then 
    AlbumArtWidth = Int(Edt5.Text)
  End If
  If IsNumeric(Edt6.Text) Then 
    AlbumArtIcon = Int(Edt6.Text)
  End If
  If IsNumeric(Edt13.Text) Then 
    ScreenWidth = Int(Edt13.Text)
  End If
  If Not Edt15.Text = "" Then 
    UnknownArtist = Edt15.Text
  End If
  If Not Edt16.Text = "" Then 
    UnknownAlbum = Edt16.Text
  End If
  GroupNonAlpha = Edt17.Checked
  AlbumArtists = Edt18.Checked
  IgnoreThes = Edt19.Checked
  SupportUnicode = Edt20.Checked
  ApplyMask = Edt21.Checked
  SeparateFiles = False
  FramedView = False      
  FramedMode = 0
  Select Case Edt14.ItemIndex
    Case 1
      SeparateFiles = True
    Case 2
      FramedView = True
    Case 3
      FramedView = True
      FramedMode = 1
  End Select
  AlbumsByYear = Edt23.Checked
  NewImages = Edt24.Checked
  If Not Edt25.Text = "" Then 
    PageTitle = Edt25.Text
  End If
  ListGenres = Edt26.Checked
  IncludeMode = Edt28.ItemIndex
  FilterName = Edt27.ItemText(Edt27.ItemIndex)
  FilterSQL = GetFilterSQL(FilterName)
  ShowUnknown = Edt29.Checked
  CompTrackMask = Edt30.Text
  CompAlbumMask = Edt31.Text  
  AlbumsPerRow = Edt32.Value
  Playlists = Edt33.Checked
  DisplayTop = Edt34.ItemIndex
  TrackLinks = Edt35.ItemIndex
  ListYears = Edt36.Checked
  TooltipMask = Edt37.Text
  TableHeader = Edt38.Checked
  ArtistMask = Edt39.Text
  ListArtGenres = Edt40.Checked
  ListArtYears = Edt41.Checked
  ShowLinks = Edt42.Checked
  ShowTracks = Edt43.Checked
  ShowStats = Edt44.Checked 
  DetailsMask = Edt45.Text
  RecentLimit = Edt46.Value
  PlayedLimit = Edt47.Value
  RatedLimit = Edt48.Value  
  
  'selected tracks
  Dim i : i = 0
  Dim j : j = 0
  Dim sql : sql = ""
  If Edt27.ItemIndex = 2 Then
    Dim sell : Set sell = SDB.SelectedSongList
    Dim selv : selv = ""
    sql = "CREATE TEMP TABLE IF NOT EXISTS tmpSelected (tmpIDSong INTEGER PRIMARY KEY)"
    If Debug Then
      Call out("#"&sql)
    End If
    SDB.Database.ExecSql(sql)
    sql = "DELETE FROM tmpSelected"
    If Debug Then
      Call out("#"&sql)
    End If    
    SDB.Database.ExecSql(sql)
    For i = 0 To sell.Count-1
      j = j+1
      selv = selv&"UNION SELECT "&sell.item(i).ID&" FROM (SELECT 1) "
      If j = 100 Then 
        sql = "REPLACE INTO tmpSelected (tmpIDSong) "&Mid(selv,7)
        If Debug Then
          Call out("#"&sql)
        End If        
        SDB.Database.ExecSql(sql)
        selv = ""
        j = 0
      End If
    Next
    If Not (selv = "") Then 
      sql = "REPLACE INTO tmpSelected (tmpIDSong) "&Mid(selv,7)
      If Debug Then
        Call out("#"&sql)
      End If
      SDB.Database.ExecSql(sql)
    End If
    Set sell = Nothing  
  End If
  
  'save settings
  UpdateProgress("Saving settings...")
  ini.StringValue("AdvancedReport","Path") = Path
  ini.StringValue("AdvancedReport","TrackMask") = TrackMask
  ini.StringValue("AdvancedReport","AlbumMask") = AlbumMask
  ini.StringValue("AdvancedReport","AlbumArtMode") = AlbumArtMode
  ini.StringValue("AdvancedReport","AlbumArtWidth") = AlbumArtWidth
  ini.StringValue("AdvancedReport","AlbumArtIcon") = AlbumArtIcon
  ini.StringValue("AdvancedReport","ScreenWidth") = ScreenWidth
  ini.StringValue("AdvancedReport","UnknownArtist") = UnknownArtist
  ini.StringValue("AdvancedReport","UnknownAlbum") = UnknownAlbum 
  ini.StringValue("AdvancedReport","SeparateFiles") = Edt14.ItemIndex
  If GroupNonAlpha Then
    ini.StringValue("AdvancedReport","GroupNonAlpha") = "1"
  Else
    ini.StringValue("AdvancedReport","GroupNonAlpha") = "0"
  End If
  If AlbumArtists Then
    ini.StringValue("AdvancedReport","AlbumArtists") = "1"
  Else
    ini.StringValue("AdvancedReport","AlbumArtists") = "0"
  End If  
  If IgnoreThes Then
    ini.StringValue("AdvancedReport","IgnoreThes") = "1"
  Else
    ini.StringValue("AdvancedReport","IgnoreThes") = "0"
  End If
  If SupportUnicode Then
    ini.StringValue("AdvancedReport","SupportUnicode") = "1"
  Else
    ini.StringValue("AdvancedReport","SupportUnicode") = "0"
  End If
  If ApplyMask Then
    ini.StringValue("AdvancedReport","ApplyMask") = "1"
  Else
    ini.StringValue("AdvancedReport","ApplyMask") = "0"
  End If  
  If AlbumsByYear Then
    ini.StringValue("AdvancedReport","AlbumsByYear") = "1"
  Else
    ini.StringValue("AdvancedReport","AlbumsByYear") = "0"
  End If  
  If NewImages Then
    ini.StringValue("AdvancedReport","NewImages") = "1"
  Else
    ini.StringValue("AdvancedReport","NewImages") = "0"
  End If   
  ini.StringValue("AdvancedReport","PageTitle") = PageTitle
  If ListGenres Then
    ini.StringValue("AdvancedReport","ListGenres") = "1"
  Else
    ini.StringValue("AdvancedReport","ListGenres") = "0"
  End If   
  ini.StringValue("AdvancedReport","FilterName") = FilterName
  If ShowUnknown Then
    ini.StringValue("AdvancedReport","ShowUnknown") = "1"
  Else
    ini.StringValue("AdvancedReport","ShowUnknown") = "0"
  End If    
  ini.StringValue("AdvancedReport","IncludeMode") = IncludeMode
  ini.StringValue("AdvancedReport","CompTrackMask") = CompTrackMask
  ini.StringValue("AdvancedReport","CompAlbumMask") = CompAlbumMask  
  ini.StringValue("AdvancedReport","AlbumsPerRow") = AlbumsPerRow  
  If Playlists Then
    ini.StringValue("AdvancedReport","Playlists") = "1"
  Else
    ini.StringValue("AdvancedReport","Playlists") = "0"
  End If   
  ini.StringValue("AdvancedReport","DisplayTop") = DisplayTop
  ini.StringValue("AdvancedReport","TrackLinks") = TrackLinks
  If ListYears Then
    ini.StringValue("AdvancedReport","ListYears") = "1"
  Else
    ini.StringValue("AdvancedReport","ListYears") = "0"
  End If     
  ini.StringValue("AdvancedReport","TooltipMask") = TooltipMask
  If TableHeader Then
    ini.StringValue("AdvancedReport","TableHeader") = "1"
  Else
    ini.StringValue("AdvancedReport","TableHeader") = "0"
  End If
  ini.StringValue("AdvancedReport","ArtistMask") = ArtistMask
  If ListArtGenres Then
    ini.StringValue("AdvancedReport","ListArtGenres") = "1"
  Else
    ini.StringValue("AdvancedReport","ListArtGenres") = "0"
  End If   
  If ListArtYears Then
    ini.StringValue("AdvancedReport","ListArtYears") = "1"
  Else
    ini.StringValue("AdvancedReport","ListArtYears") = "0"
  End If
  If ShowLinks Then
    ini.StringValue("AdvancedReport","ShowLinks") = "1"
  Else
    ini.StringValue("AdvancedReport","ShowLinks") = "0"
  End If
  If ShowTracks Then
    ini.StringValue("AdvancedReport","ShowTracks") = "1"
  Else
    ini.StringValue("AdvancedReport","ShowTracks") = "0"
  End If
  If ShowStats Then
    ini.StringValue("AdvancedReport","ShowStats") = "1"
  Else
    ini.StringValue("AdvancedReport","ShowStats") = "0"
  End If
  ini.StringValue("AdvancedReport","DetailsMask") = DetailsMask
  ini.StringValue("AdvancedReport","RecentLimit") = RecentLimit
  ini.StringValue("AdvancedReport","PlayedLimit") = PlayedLimit
  ini.StringValue("AdvancedReport","RatedLimit") = RatedLimit                                    
  
  'cancelled by user
  If FilterSQL = "~EXIT~" Then
    Exit Sub
  End If
  
  'create logfile
  If Debug Then
    Call clear()
  End If  
  ShowIcons = ini.StringValue("AdvancedReport","ShowIcons")
  
  'calcuate summary
  UpdateProgress("Calculating summary...")
  If FilterSQL = "" Then
    FromClause1  = ""
    WhereClause1 = ""
    FromClause2  = ""
    WhereClause2 = ""
  Else
    FromClause1  = ",Songs,ArtistsAlbums"
    WhereClause1 = "AND Songs.IDAlbum=ArtistsAlbums.IDAlbum AND ArtistsAlbums.IDArtist=Artists.ID "&FilterSQL
    FromClause2  = ",Songs,ArtistsSongs"
    WhereClause2 = "AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.IDArtist=Artists.ID AND ArtistsSongs.PersonType=1 "&FilterSQL
  End If
  If AlbumArtists Then
    sql = "SELECT Count(DISTINCT Artists.ID) As Nombre FROM Artists"&FromClause1&" WHERE Artists.Albums>0 "&WhereClause1
  Else
    sql = "SELECT Count(DISTINCT Artists.ID) As Nombre FROM Artists"&FromClause2&" WHERE Artists.Tracks>0 "&WhereClause2
  End If
  If Debug Then
    Call out("#"&sql)
  End If
  Dim iter : Set iter = SDB.Database.OpenSQL(sql)
  Dim artists : artists = CLng(iter.StringByName("Nombre"))
  If FilterSQL = "" Then
    FromClause1 = ""
    WhereClause1 = "Albums.Album!=''"
  Else
    FromClause1 = ", Songs "
    WhereClause1 = "Albums.Album!='' AND Songs.IDAlbum=Albums.ID "&FilterSQL
  End If
  sql = "SELECT Count(DISTINCT Albums.ID) AS Nombre FROM Albums "&FromClause1&"WHERE "&WhereClause1
  If Debug Then
    Call out("#"&sql)
  End If  
  Set iter = SDB.Database.OpenSQL(sql)
  Dim albums : albums = CLng(iter.StringByName("Nombre"))
  sql = "SELECT Count(DISTINCT GenresSongs.IDGenre) AS Nombre FROM GenresSongs,Songs WHERE Songs.ID=GenresSongs.IDSong "&FilterSQL
  If Debug Then
    Call out("#"&sql)
  End If  
  Set iter = SDB.Database.OpenSQL(sql)
  Dim genres : genres = CLng(iter.StringByName("Nombre"))
  sql = "SELECT Count(*) AS Nombre FROM Songs WHERE ID>0 "&FilterSQL
  If Debug Then
    Call out("#"&sql)
  End If  
  Set iter = SDB.Database.OpenSQL(sql)
  Dim tracks : tracks = CLng(iter.StringByName("Nombre"))  
  Dim unknown : unknown = 0
  If ShowTracks And ShowUnknown Then
    sql = "SELECT Count(*) FROM (SELECT ArtistsSongs.IDArtist FROM Songs,ArtistsSongs WHERE Songs.Album='' AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.PersonType=1 "&FilterSQL&"GROUP BY ArtistsSongs.IDArtist)"
    If Debug Then
      Call out("#"&sql)
    End If    
    Set iter = SDB.Database.OpenSQL(sql)
    unknown = CLng(iter.StringByIndex(0))
  End If
  Dim plays : plays = 0
  If Playlists Then
    sql = "SELECT Count(*) FROM Playlists"
    If Debug Then
      Call out("#"&sql)
    End If    
    Set iter = SDB.Database.OpenSQL(sql)
    plays = CLng(iter.StringByIndex(0))
  End If
  If ShowTracks Then
    Progress.MaxValue = artists+artists+albums+albums+unknown+((plays+1)*3)
  Else
    Progress.MaxValue = artists+artists+3    
  End If
  Progress.Value = 0
  
  'initialise images
  UpdateProgress("Copying images...")
  BaseDir = fso.GetParentFolderName(Path)
  ArtsDir = BaseDir&"\artists\"
  AlbsDir = BaseDir&"\albums\"
  ImgsDir = BaseDir&"\images\"
  StylDir = BaseDir&"\css\"  
  PlayDir = BaseDir&"\playlists\"
  Call GeneratePath(ImgsDir)
  Call GeneratePath(StylDir)  
  If Not (fso.FileExists(ImgsDir&"none.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"none.jpg") Then
      fso.CopyFile SDB.ScriptsPath&"none.jpg",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"star.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"star.png") Then
      fso.CopyFile SDB.ScriptsPath&"star.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"half-star2.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"half-star2.png") Then
      fso.CopyFile SDB.ScriptsPath&"half-star2.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"no-star2.jpg")) Then    
    If fso.FileExists(SDB.ScriptsPath&"no-star2.png") Then
      fso.CopyFile SDB.ScriptsPath&"no-star2.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"no-star.jpg")) Then    
    If fso.FileExists(SDB.ScriptsPath&"no-star.png") Then
      fso.CopyFile SDB.ScriptsPath&"no-star.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"bomb.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"bomb.png") Then
      fso.CopyFile SDB.ScriptsPath&"bomb.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"harddisk.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"harddisk.png") Then
      fso.CopyFile SDB.ScriptsPath&"harddisk.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"usbpen.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"usbpen.png") Then
      fso.CopyFile SDB.ScriptsPath&"usbpen.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"ipod.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"ipod.png") Then
      fso.CopyFile SDB.ScriptsPath&"ipod.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"server.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"server.png") Then
      fso.CopyFile SDB.ScriptsPath&"server.png",ImgsDir,True
    End If
  End If
  If Not (fso.FileExists(ImgsDir&"cd-dvd.jpg")) Then
    If fso.FileExists(SDB.ScriptsPath&"cd-dvd.png") Then
      fso.CopyFile SDB.ScriptsPath&"cd-dvd.png",ImgsDir,True
    End If
  End If          
  If Not (fso.FileExists(StylDir&"report.css")) Then    
    If fso.FileExists(SDB.ScriptsPath&"report.css") Then
      fso.CopyFile SDB.ScriptsPath&"report.css",StylDir, True
    End If  
  End If    
  If SeparateFiles Or FramedView Then
    Call GeneratePath(ArtsDir)
    If ShowTracks Then
      Call GeneratePath(AlbsDir)
    End If
    If Playlists Then
      Call GeneratePath(PlayDir)
    End If
    ImageBase = "../images/"
  Else
    ImageBase = "images/"
  End If  
  Set Artwork = CreateObject("Scripting.Dictionary")  
  
  'create output file
  UpdateProgress("Creating output file...")
  Dim fout : Set fout = fso.CreateTextFile(Path,True,SupportUnicode)
  Dim ext : ext = Mid(Path,InStrRev(Path,"."))
  ini.StringValue("AdvancedReport","Extension") = ext  
  If FramedView Then
    Dim w : w = 280
    ScreenWidth = ScreenWidth - w
    fout.WriteLine "<!DOCTYPE html PUBLIC ""-//W3C//DTD XHTML 1.0 Frameset//EN"" ""http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd"">"
    fout.WriteLine "<html>"
    fout.WriteLine "<head>"
    fout.WriteLine "  <title>"&PageTitle&"</title>"
    fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">"
    fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
    fout.WriteLine "  <link rel=""stylesheet"" href=""css/report.css"" type=""text/css"">"
    fout.WriteLine "</head>" 
    Select Case FramedMode
      Case 0 
        fout.WriteLine "<frameset cols="""&w&",*"">"
        fout.WriteLine "  <frameset rows="""&w&",*"">"
        fout.WriteLine "    <frame src=""f_caps"&ext&""" name=""caps"" frameborder=""1"">"
        fout.WriteLine "    <frame src=""f_list"&ext&""" name=""list"" frameborder=""1"">"
        fout.WriteLine "  </frameset>"
        If ShowStats Then
          fout.WriteLine "  <frame src=""f_stat"&ext&""" name=""main"" frameborder=""1"">"
        Else
          fout.WriteLine "  <frame src=""about:blank"" name=""main"" frameborder=""1"">"
        End If
      Case 1
        fout.WriteLine "<frameset rows="""&Int(w*0.6)&",*"">"
        fout.WriteLine "  <frame src=""f_caps"&ext&""" name=""caps"" frameborder=""1"">"
        fout.WriteLine "  <frameset cols="""&w&",*"">"
        fout.WriteLine "    <frame src=""f_list"&ext&""" name=""list"" frameborder=""1"">"
        If ShowStats Then
          fout.WriteLine "    <frame src=""f_stat"&ext&""" name=""main"" frameborder=""1"">"
        Else        
          fout.WriteLine "    <frame src=""about:blank"" name=""main"" frameborder=""1"">"
        End If        
        fout.WriteLine "  </frameset>"
    End Select
    fout.WriteLine "  <noframes>"
    fout.WriteLine "    <body>"
    fout.WriteLine "      <h1>This browser does not support frames!</h1>"
    fout.WriteLine "    </body>"
    fout.WriteLine "  </noframes>"
    fout.WriteLine "</frameset>"    
    fout.WriteLine "</html>"
    fout.Close
    Set fout = fso.CreateTextFile(BaseDir&"\f_caps"&ext,True,SupportUnicode)
  End If
  
  fout.WriteLine "<!DOCTYPE html>"
  fout.WriteLine "<html>"
  fout.WriteLine "<head>"
  fout.WriteLine "  <title>"&PageTitle&"</title>"
  fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">"
  fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
  fout.WriteLine "  <link rel=""stylesheet"" href=""css/report.css"" type=""text/css"">"
  fout.WriteLine "</head>"
  fout.WriteLine "<body>" 
  fout.WriteLine "  <div class=""head_wrapper"">" 
  fout.WriteLine "    <h1>"
  fout.WriteLine "      <a id=""top"" href=""http://www.mediamonkey.com"">"&PageTitle&"</a>"
  fout.WriteLine "    </h1>"
  fout.WriteLine "    <p>"
  fout.WriteLine "      Created: "&Date&" "&Time&" using '"&AppName&"' by <a href=""http://trixmoto.net"" target=""_blank"">Trixmoto</a>"
  Dim brs : brs = "<br>"
  If (FramedView) And (FramedMode = 1) Then
    brs = " - "
    fout.WriteLine "      "&brs
  Else
    fout.WriteLine "      "&brs&brs
  End If
  If AlbumArtists Then
    fout.WriteLine "      Album artists: "&artists&brs
  Else
    fout.WriteLine "      Artists: "&artists&brs
  End If
  fout.WriteLine "      Albums: "&albums&brs
  fout.WriteLine "      Genres: "&genres&brs  
  If Playlists Then
    fout.WriteLine "      Tracks: "&tracks&brs
    fout.WriteLine "      Playlists: "&plays
  Else
    fout.WriteLine "      Tracks: "&tracks
  End If
  fout.WriteLine "    </p>"
 
  'output a-z list
  UpdateProgress("Writing contents list...")
  Dim comp : Set comp = CreateObject("Scripting.Dictionary")
  Dim cmp : cmp = ""
  Dim cap : cap = ""
  Dim href : href = ""
  Dim targ : targ = ""
  Dim NotYet : NotYet = True
  If (FramedView) And (FramedMode = 1) Then
    fout.WriteLine "    <h2 style=""display:inline;float:left;padding:0px;margin:0px;"">Contents:&nbsp;&nbsp;</h2>"
  Else
    fout.WriteLine "    <h2>Contents</h2>"
  End If
  fout.WriteLine "    <ul>"
  If FilterSQL = "" Then
    FromClause1  = ""
    FromClause2  = ""
    WhereClause1 = ""
    WhereClause2 = ""
  Else
    FromClause1  = ", Songs, ArtistsAlbums "
    WhereClause1 = "AND Songs.IDAlbum=ArtistsAlbums.IDAlbum AND ArtistsAlbums.IDArtist=Artists.ID "&FilterSQL
    FromClause2  = ", Songs, ArtistsSongs "
    WhereClause2 = "AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.IDArtist=Artists.ID AND ArtistsSongs.PersonType IN (1,3) "&FilterSQL
  End If    
  If AlbumArtists Then
    sql = "SELECT DISTINCT UpperW(SubStr(Artists.Artist,1,1)) FROM Artists "&FromClause1&"WHERE Artists.Albums>0 "&WhereClause1&"ORDER BY UpperW(SubStr(Artists.Artist,1,1))"
  Else
    sql = "SELECT DISTINCT UpperW(SubStr(Artists.Artist,1,1)) FROM Artists "&FromClause2&"WHERE (Artists.Tracks>0 OR Artists.Albums>0) "&WhereClause2&"ORDER BY UpperW(SubStr(Artists.Artist,1,1))"
  End If
  If FramedView Then
    href = "f_list"&ext
    targ = " target=""list"""
  End If
  If Debug Then
    Call out("#"&sql)
  End If  
  Set iter = SDB.Database.OpenSQL(sql)
  Do While Not iter.EOF
    cap = UCase(iter.StringByIndex(0))
    If GroupNonAlpha Then
      If InStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ",cap) = 0 Then
        If NotYet Then
          fout.WriteLine "      <li><a href="""&href&"#hash"" accesskey=""#"""&targ&">#</a></li>"
          NotYet = False
        End If
      Else
        fout.WriteLine "      <li><a href="""&href&"#"&SDB.toASCII(cap)&""" accesskey="""&SDB.toASCII(cap)&""""&targ&">"&MapXML(cap)&"</a></li>"
      End If
    Else
      fout.WriteLine "      <li><a href="""&href&"#"&SDB.toASCII(cap)&""" accesskey="""&SDB.toASCII(cap)&""""&targ&">"&MapXML(cap)&"</a></li>"
    End If
    iter.Next
  Loop
  If Playlists Then
    href = "#play"
    If SeparateFiles Then 
      href = "playlists/play"&ext
    End If
    If FramedView Then 
      href = "playlists/play"&ext&""" target=""main"
    End If  
    fout.WriteLine "      <li><a href="""&href&""">Playlists</a></li>"
  End If
  If ShowStats Then
    href = "#stat"
    If SeparateFiles Then 
      href = "f_stat"&ext
    End If
    If FramedView Then 
      href = "f_stat"&ext&""" target=""main"
    End If  
    fout.WriteLine "      <li><a href="""&href&""">Top albums</a></li>"
  End If          
  fout.WriteLine "    </ul>"
  fout.WriteLine "  </div>"
  
  'output top albums
  If ShowStats Then
    UpdateProgress("Selecting top albums...")
    Dim name3,id3,td3
    If FramedView Then
      fout.WriteLine "</body>"
      fout.WriteLine "</html>"
      fout.Close
	
      Set fout = fso.CreateTextFile(BaseDir&"\f_stat"&ext,True,SupportUnicode)
      fout.WriteLine "<!DOCTYPE html>"
      fout.WriteLine "<html>"
      fout.WriteLine "<head>"
      fout.WriteLine "  <title>"&PageTitle&"</title>"
      fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">"
      fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
      fout.WriteLine "  <link rel=""stylesheet"" href=""css/report.css"" type=""text/css"">"
      fout.WriteLine "</head>"
      fout.WriteLine "<body>"
      fout.WriteLine "  <a id=""top"">&nbsp;</a>" 
    Else
      fout.WriteLine "  <br><br>"
    End If    
    fout.WriteLine "  <div class=""stat_wrapper"">" 
    
    'top added
    If RecentLimit > 0 Then
      fout.WriteLine "    <h2>Recently added</h2>"
      fout.WriteLine "    <table title=""Recently added"">"
      fout.WriteLine "      <tr>" 
      j = 1     
      sql = "SELECT Songs.IDAlbum,Albums.Album FROM Songs,Albums WHERE Albums.ID>0 AND Albums.Album!='' AND Albums.Tracks>4 AND Albums.ID=Songs.IDAlbum "&FilterSQL&"GROUP BY Songs.IDAlbum,Albums.Album ORDER BY Avg(Songs.DateAdded) Desc LIMIT "&RecentLimit
      If Debug Then
        Call out("#"&sql)
      End If  
      Set iter = SDB.Database.OpenSQL(sql)
      Do While Not iter.EOF
        SDB.ProcessMessages
        If Progress.Terminate Then 
          Exit Do
        End If
        name3 = iter.StringByIndex(1)
        id3 = iter.StringByIndex(0)
        href = "#album_"&id3
        If SeparateFiles Then 
          href = "albums/album_"&id3&ext
        End If
        If FramedView Then 
          href = "albums/album_"&id3&ext&""" target=""main"
        End If
        cmp = ""
        If Not (CompAlbumMask = "") Or Not (CompTrackMask = "") Then
          sql = "SELECT Count(DISTINCT CASE WHEN InStr(Artist,';')>0 THEN SubStr(Artist,1,InStr(Artist,';')-1) WHEN InStr(Artist,';')=0 THEN Artist END) COLLATE IUNICODE FROM Songs WHERE IDAlbum="&id3
          If Debug Then
            Call out("#"&sql)
          End If      
          Set dit = SDB.Database.OpenSQL(sql)
          If dit.ValueByIndex(0) > 1 Then
            cmp = "Y"
          End If        
          comp.Item("#"&id3) = cmp
        End If
        If ApplyMask Then 
          name3 = GetAlbum(id3,0,ext,cmp)
        Else
          name3 = MapXML(name3)
        End If    
        td3 = "</td>"
        fout.WriteLine "        <td class=""album"" style=""vertical-align:bottom;"">"
        If DisplayTop = 0 Then
          If ShowTracks Then
            fout.WriteLine "        <p><a href="""&href&""">"&name3&"</a></p>"
          Else
            fout.WriteLine "        <p>"&name3&"</p>"
          End If
        Else
          If ShowTracks Then
            td3 = "<p><a href="""&href&""">"&name3&"</a></p></td>"
          Else
            td3 = "<p>"&name3&"</p></td>"
          End If
        End If
        If AlbumArtMode < 3 Then
          If AlbumArtIcon > 0 Then
            If ShowTracks Then
              fout.WriteLine "        <a href="""&href&"""><img src="""&Replace(GetAlbumArt(id3),"../","")&""" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image""></a>"&td3
            Else
              fout.WriteLine "        <img src="""&Replace(GetAlbumArt(id3),"../","")&""" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image"">"&td3
            End If
            j = j + 1
            If (AlbumsPerRow = 0 And j > ScreenWidth/(AlbumArtIcon+25)) Or (AlbumsPerRow > 0 And j > AlbumsPerRow) Then
              fout.WriteLine "      </tr>"
              fout.WriteLine "      <tr>"
              j = 1
            End If  
          Else
            fout.WriteLine "        "&td3
            fout.WriteLine "      </tr>"
            fout.WriteLine "      <tr>"
          End If
        Else
          fout.WriteLine "        "&td3
          fout.WriteLine "      </tr>"
          fout.WriteLine "      <tr>"
        End If      
        iter.Next
      Loop
      fout.WriteLine "        <td>&nbsp;</td>"        
      fout.WriteLine "      </tr>"
      fout.WriteLine "    </table>"
    End If    
    
    'top played
    If PlayedLimit > 0 Then     
      fout.WriteLine "    <h2>Top played</h2>"
      fout.WriteLine "    <table title=""Top played"">"
      fout.WriteLine "      <tr>" 
      j = 1     
      sql = "SELECT Songs.IDAlbum,Albums.Album FROM Songs,Albums WHERE Albums.ID>0 AND Albums.Album!='' AND Albums.Tracks>4 AND Songs.Playcounter>0 AND Albums.ID=Songs.IDAlbum "&FilterSQL&"GROUP BY Songs.IDAlbum,Albums.Album ORDER BY Avg(Songs.PlayCounter) Desc LIMIT "&PlayedLimit
      If Debug Then
        Call out("#"&sql)
      End If  
      Set iter = SDB.Database.OpenSQL(sql)
      Do While Not iter.EOF
        SDB.ProcessMessages
        If Progress.Terminate Then 
          Exit Do
        End If      
        name3 = iter.StringByIndex(1)
        id3 = iter.StringByIndex(0)
        href = "#album_"&id3
        If SeparateFiles Then 
          href = "albums/album_"&id3&ext
        End If
        If FramedView Then 
          href = "albums/album_"&id3&ext&""" target=""main"
        End If
        cmp = ""
        If Not (CompAlbumMask = "") Or Not (CompTrackMask = "") Then
          sql = "SELECT Count(DISTINCT CASE WHEN InStr(Artist,';')>0 THEN SubStr(Artist,1,InStr(Artist,';')-1) WHEN InStr(Artist,';')=0 THEN Artist END) COLLATE IUNICODE FROM Songs WHERE IDAlbum="&id3
          If Debug Then
            Call out("#"&sql)
          End If      
          Set dit = SDB.Database.OpenSQL(sql)
          If dit.ValueByIndex(0) > 1 Then
            cmp = "Y"
          End If        
          comp.Item("#"&id3) = cmp
        End If
        If ApplyMask Then 
          name3 = GetAlbum(id3,0,ext,cmp)
        Else
          name3 = MapXML(name3)
        End If    
        td3 = "</td>"
        fout.WriteLine "        <td class=""album"" style=""vertical-align:bottom;"">"
        If DisplayTop = 0 Then
          If ShowTracks Then
            fout.WriteLine "        <p><a href="""&href&""">"&name3&"</a></p>"
          Else
            fout.WriteLine "        <p>"&name3&"</p>"
          End If
        Else
          If ShowTracks Then
            td3 = "<p><a href="""&href&""">"&name3&"</a></p></td>"
          Else
            td3 = "<p>"&name3&"</p></td>"
          End If
        End If
        If AlbumArtMode < 3 Then
          If AlbumArtIcon > 0 Then
            If ShowTracks Then
              fout.WriteLine "        <a href="""&href&"""><img src="""&Replace(GetAlbumArt(id3),"../","")&""" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image""></a>"&td3
            Else
              fout.WriteLine "        <img src="""&Replace(GetAlbumArt(id3),"../","")&""" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image"">"&td3
            End If
            j = j + 1
            If (AlbumsPerRow = 0 And j > ScreenWidth/(AlbumArtIcon+25)) Or (AlbumsPerRow > 0 And j > AlbumsPerRow) Then
              fout.WriteLine "      </tr>"
              fout.WriteLine "      <tr>"
              j = 1
            End If  
          Else
            fout.WriteLine "        "&td3
            fout.WriteLine "      </tr>"
            fout.WriteLine "      <tr>"
          End If
        Else
          fout.WriteLine "        "&td3
          fout.WriteLine "      </tr>"
          fout.WriteLine "      <tr>"
        End If
        iter.Next
      Loop
      fout.WriteLine "        <td>&nbsp;</td>"        
      fout.WriteLine "      </tr>"
      fout.WriteLine "    </table>"        
    End If
    
    'top rated
    If RatedLimit > 0 Then
      fout.WriteLine "    <h2>Top rated</h2>"
      fout.WriteLine "    <table title=""Top rated"">"
      fout.WriteLine "      <tr>" 
      j = 1         
      sql = "SELECT Songs.IDAlbum,Albums.Album FROM Songs,Albums WHERE Albums.ID>0 AND Albums.Album!='' AND Albums.Tracks>4 AND Albums.ID=Songs.IDAlbum "&FilterSQL&"GROUP BY Songs.IDAlbum,Albums.Album ORDER BY Avg(Songs.Rating) Desc LIMIT "&RatedLimit
      If Debug Then
        Call out("#"&sql)
      End If  
      Set iter = SDB.Database.OpenSQL(sql)
      Do While Not iter.EOF
        SDB.ProcessMessages
        If Progress.Terminate Then 
          Exit Do
        End If      
        name3 = iter.StringByIndex(1)
        id3 = iter.StringByIndex(0)
        href = "#album_"&id3
        If SeparateFiles Then 
          href = "albums/album_"&id3&ext
        End If
        If FramedView Then 
          href = "albums/album_"&id3&ext&""" target=""main"
        End If
        cmp = ""
        If Not (CompAlbumMask = "") Or Not (CompTrackMask = "") Then
          sql = "SELECT Count(DISTINCT CASE WHEN InStr(Artist,';')>0 THEN SubStr(Artist,1,InStr(Artist,';')-1) WHEN InStr(Artist,';')=0 THEN Artist END) COLLATE IUNICODE FROM Songs WHERE IDAlbum="&id3
          If Debug Then
            Call out("#"&sql)
          End If      
          Set dit = SDB.Database.OpenSQL(sql)
          If dit.ValueByIndex(0) > 1 Then
            cmp = "Y"
          End If        
          comp.Item("#"&id3) = cmp
        End If
        If ApplyMask Then 
          name3 = GetAlbum(id3,0,ext,cmp)
        Else
          name3 = MapXML(name3)
        End If    
        td3 = "</td>"
        fout.WriteLine "        <td class=""album"" style=""vertical-align:bottom;"">"
        If DisplayTop = 0 Then
          If ShowTracks Then
            fout.WriteLine "        <p><a href="""&href&""">"&name3&"</a></p>"
          Else
            fout.WriteLine "        <p>"&name3&"</p>"
          End If
        Else
          If ShowTracks Then
            td3 = "<p><a href="""&href&""">"&name3&"</a></p></td>"
          Else
            td3 = "<p>"&name3&"</p></td>"
          End If
        End If
        If AlbumArtMode < 3 Then
          If AlbumArtIcon > 0 Then
            If ShowTracks Then
              fout.WriteLine "        <a href="""&href&"""><img src="""&Replace(GetAlbumArt(id3),"../","")&""" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image""></a>"&td3
            Else
              fout.WriteLine "        <img src="""&Replace(GetAlbumArt(id3),"../","")&""" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image"">"&td3
            End If
            j = j + 1
            If (AlbumsPerRow = 0 And j > ScreenWidth/(AlbumArtIcon+25)) Or (AlbumsPerRow > 0 And j > AlbumsPerRow) Then
              fout.WriteLine "      </tr>"
              fout.WriteLine "      <tr>"
              j = 1
            End If  
          Else
            fout.WriteLine "        "&td3
            fout.WriteLine "      </tr>"
            fout.WriteLine "      <tr>"
          End If
        Else
          fout.WriteLine "        "&td3
          fout.WriteLine "      </tr>"
          fout.WriteLine "      <tr>"
        End If
        iter.Next
      Loop        
      fout.WriteLine "        <td>&nbsp;</td>"        
      fout.WriteLine "      </tr>"
      fout.WriteLine "    </table>"
    End If    
    
    fout.WriteLine "    <br>"
    fout.WriteLine "  </div>"    
  End If
  
  'ignore prefixes
  Dim Field : Field = "UpperW(Artists.Artist)"
  Dim thea : thea = Array()
  Dim OrderField : OrderField = ""
  If IgnoreThes Then
    If ini.BoolValue("Options","IgnoreTHEs") Then
      thea = Split(ini.StringValue("Options","IgnoreTHEStrings"),",")
      For i = 0 To UBound(thea)
        temp = UCase(Trim(thea(i)))
        OrderField = OrderField&"WHEN SubStr("&Field&",1,"&(Len(temp)+1)&") = '"&Replace(temp,"'","''")&" ' THEN SubStr("&Field&","&(Len(temp)+2)&") "
      Next
    End If
  End If
  If Len(OrderField) = 0 Then
    OrderField = Field
  Else
    OrderField = "CASE "&OrderField&"ELSE "&Field&" END"
  End If    
  
  'show links
  Dim alblnk : Set alblnk = CreateObject("Scripting.Dictionary")
  Dim artlnk : Set artlnk = CreateObject("Scripting.Dictionary")
  If (ShowLinks) And (fso.FileExists(SDB.ScriptsPath&"\Auto\WebSearch.xml")) Then
    Dim xml,par,kid,cfg,cpt,url
    Set xml = CreateObject("Microsoft.XMLDOM")
    xml.async = "false"
    Call xml.load(SDB.ScriptsPath&"\Auto\WebSearch.xml")
    For Each kid In xml.getElementsByTagName("site")
      cpt = ""
      url = ""
      For Each cfg In kid.childNodes
        If cfg.baseName = "caption" Then
          cpt = cfg.Text
        End If
        If cfg.baseName = "url" Then
          url = cfg.Text
        End If
      Next
      If (cpt <> "") And (url <> "") And (InStr(url,"%title%") = 0) Then
        If InStr(url,"%album%") > 0 Then
          alblnk.Item(cpt) = url
        Else
          If (InStr(url,"%artist%") > 0) Or (InStr(url,"%albumartist%") > 0) Then
            artlnk.Item(cpt) = url
          End If
        End If      
      End If
    Next
    Set xml = Nothing  
  End If  
  
  'output artists
  If FramedView Then
    fout.WriteLine "</body>"
    fout.WriteLine "</html>"
    fout.Close
	
    Set fout = fso.CreateTextFile(BaseDir&"\f_list"&ext,True,SupportUnicode)
    fout.WriteLine "<!DOCTYPE html>"
    fout.WriteLine "<html>"
    fout.WriteLine "<head>"
    fout.WriteLine "  <title>"&PageTitle&"</title>"
    fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">"
    fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
    fout.WriteLine "  <link rel=""stylesheet"" href=""css/report.css"" type=""text/css"">"
    fout.WriteLine "</head>"
    fout.WriteLine "<body>"
    fout.WriteLine "  <a id=""top"">&nbsp;</a>" 
  Else
    fout.WriteLine "  <br><br>"
  End If    
  fout.WriteLine "  <div class=""nav_wrapper"">"
  fout.WriteLine "    <h2>Artists</h2>"  
  Dim name,id,MaybeCap
  cap = ""
  NotYet = True    
  If FilterSQL = "" Then
    FromClause1  = ""
    FromClause2  = ""
    WhereClause1 = ""
    WhereClause2 = ""
  Else
    FromClause1  = ",Songs,ArtistsAlbums"
    WhereClause1 = "AND Songs.IDAlbum=ArtistsAlbums.IDAlbum AND ArtistsAlbums.IDArtist=Artists.ID "&FilterSQL
    FromClause2  = ",Songs,ArtistsSongs"
    WhereClause2 = "AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.IDArtist=Artists.ID AND ArtistsSongs.PersonType IN (1,3) "&FilterSQL
  End If
  If AlbumArtists Then
    sql = "SELECT DISTINCT Artists.Artist,Artists.ID FROM Artists"&FromClause1&" WHERE Artists.Albums>0 "&WhereClause1&"ORDER BY "&OrderField
  Else
    sql = "SELECT DISTINCT Artists.Artist,Artists.ID FROM Artists"&FromClause2&" WHERE (Artists.Tracks>0 OR Artists.Albums>0) "&WhereClause2&"ORDER BY "&OrderField
  End If
  If Debug Then
    Call out("#"&sql)
  End If  
  Set iter = SDB.Database.OpenSQL(sql)
  Do While Not iter.EOF
    SDB.ProcessMessages
    If Progress.Terminate Then 
      Exit Do
    End If
    name = iter.StringByName("Artist")
    id = iter.StringByName("ID")
    If (id = 0) Or (name = "") Then 
      name = UnknownArtist
    End If
    Progress.Increase
    UpdateProgress("Contents: "&name)
    href = "#artist_"&id
    If SeparateFiles Then 
      href = "artists/artist_"&id&ext
    End If
    If FramedView Then 
      href = "artists/artist_"&id&ext&""" target=""main"
    End If
    'ignore thes
    MaybeCap = True
    For i = 0 To UBound(thea)
      temp = UCase(Trim(thea(i)))&" "
      If UCase(Left(name,Len(temp))) = temp Then
        MaybeCap = False
      End If
    Next      
    'add new header
    If (MaybeCap) And Not (UCase(Left(name,1)) = cap) Then
      cap = UCase(Left(name,1))
      If GroupNonAlpha Then
        If InStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ",cap) = 0 Then
          If NotYet Then
            fout.WriteLine "    <h2><a id=""hash"">#</a> (<a href=""#top"">top</a>)</h2>"
            NotYet = False
          End If
        Else
          fout.WriteLine "    <h2><a id="""&SDB.toASCII(cap)&""">"&MapXML(cap)&"</a> (<a href=""#top"">top</a>)</h2>"
        End If
      Else
        fout.WriteLine "    <h2><a id="""&SDB.toASCII(cap)&""">"&MapXML(cap)&"</a> (<a href=""#top"">top</a>)</h2>"
      End If      
    End If
    fout.WriteLine "    <a href="""&href&""">"&MapXML(name)&"</a><br>"
    iter.Next
  Loop
  fout.WriteLine "    <br>"
  fout.WriteLine "  </div>"
  
  Dim toplink : toplink = ""
  If SeparateFiles Or FramedView Then
    fout.WriteLine "</body>"
    fout.WriteLine "</html>"
    fout.Close  
	
    href = ArtsDir&"\artist_0"&ext
    Set fout = fso.CreateTextFile(href,True,SupportUnicode)
    fout.WriteLine "<!DOCTYPE html>"
    fout.WriteLine "<html>"
    fout.WriteLine "<head>"
    fout.WriteLine "  <title>"&UnknownArtist&"</title>"
    fout.WriteLine "</head>"
    fout.WriteLine "<body>"
    fout.WriteLine "</body>"
    fout.WriteLine "</html>"
    fout.Close
    
    If ShowTracks Then
      href = AlbsDir&"\album_0"&ext
      Set fout = fso.CreateTextFile(href,True,SupportUnicode)
      fout.WriteLine "<!DOCTYPE html>"
      fout.WriteLine "<html>"
      fout.WriteLine "<head>"
      fout.WriteLine "  <title>"&UnknownAlbum&"</title>"
      fout.WriteLine "</head>"
      fout.WriteLine "<body>"
      fout.WriteLine "</body>"
      fout.WriteLine "</html>"
      fout.Close
    End If
    
    If Playlists Then
      href = PlayDir&"\play_0"&ext
      Set fout = fso.CreateTextFile(href,True,SupportUnicode)
      fout.WriteLine "<!DOCTYPE html>"
      fout.WriteLine "<html>"
      fout.WriteLine "<head>"
      fout.WriteLine "  <title>Playlists</title>"
      fout.WriteLine "</head>"
      fout.WriteLine "<body>"
      fout.WriteLine "</body>"
      fout.WriteLine "</html>"
      fout.Close    
    End If
  Else
    toplink = " (<a href=""#top"">top</a>)"
  End If

  If Debug Then
    Call out("#"&sql)
  End If  
  Set iter = SDB.Database.OpenSQL(sql)
  Do While Not iter.EOF
    SDB.ProcessMessages
    If Progress.Terminate Then 
      Exit Do
    End If
    name = iter.StringByName("Artist")
    id = iter.StringByName("ID")
    If (id = 0) Or (name = "") Then 
      name = UnknownArtist
    End If
    Progress.Increase
    UpdateProgress("Artist: "&name)

    'create header    
    If SeparateFiles Or FramedView Then
      href = ArtsDir&"\artist_"&id&ext
      Set fout = fso.CreateTextFile(href,True,SupportUnicode) 
      fout.WriteLine "<!DOCTYPE html>"
      fout.WriteLine "<html>"
      fout.WriteLine "<head>"
      fout.WriteLine "  <title>"&MapXML(name)&"</title>" 
      fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">" 
      fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
      fout.WriteLine "  <link rel=""stylesheet"" href=""../css/report.css"" type=""text/css"">"  
      fout.WriteLine "</head>"
      fout.WriteLine "<body>"
      If SeparateFiles Then
        fout.WriteLine "  <h1>"
        fout.WriteLine "    <a href=""../"&fso.GetFileName(path)&""">"&PageTitle&"</a>"
        fout.WriteLine "  </h1>"
        fout.WriteLine "  <p>"
        fout.WriteLine "    Created: "&Date&" "&Time&" using '"&AppName&"' by <a href=""http://trixmoto.net"" target=""_blank"">Trixmoto</a>"
        fout.WriteLine "  </p>"
      End If
    End If
    fout.WriteLine "  <div class=""artist_wrapper"">"
    fout.WriteLine "    <h2><a id=""artist_"&id&""">"&GetArtist(id,name)&"</a>"&toplink&"</h2>"
    
    'list genres
    Dim iter2 : Set iter2 = Nothing
    If ListArtGenres Then
      href = ""
      sql = "SELECT DISTINCT Genres.GenreName FROM Genres,Songs,GenresSongs,ArtistsSongs WHERE GenresSongs.IDSong=Songs.ID AND GenresSongs.IDGenre=Genres.IDGenre AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.IDArtist="&id&" AND ArtistsSongs.PersonType=1 "&FilterSQL&"ORDER BY Genres.GenreName"
      If Debug Then
        Call out("#"&sql)
      End If            
      Set iter2 = SDB.Database.OpenSQL(sql)
      If Not iter2.EOF Then
        href = iter2.StringByName("GenreName")
        iter2.Next
        While Not iter2.EOF
          href = href&", "&iter2.StringByName("GenreName")
          iter2.Next
        WEnd
      End If
      fout.WriteLine "    <h3>Genre: "&MapXML(href)&"</h3>"        
    End If      
      
    'list years
    If ListArtYears Then
      href = ""
      sql = "SELECT DISTINCT Substr(Songs.Year,1,4) FROM Songs,ArtistsSongs WHERE ArtistsSongs.IDSong=Songs.ID AND ArtistsSongs.IDArtist="&id&" AND ArtistsSongs.PersonType=1 "&FilterSQL&"ORDER BY Songs.Year"
      If Debug Then
        Call out("#"&sql)
      End If            
      Set iter2 = SDB.Database.OpenSQL(sql)
      If Not iter2.EOF Then
        href = iter2.StringByIndex(0)
        iter2.Next
        While Not iter2.EOF
          href = href&", "&iter2.StringByIndex(0)
          iter2.Next
        WEnd
      End If
      Dim orig1 : orig1 = ""
      sql = "SELECT DISTINCT Substr(Songs.OrigYear,1,4) FROM Songs,ArtistsSongs WHERE ArtistsSongs.IDSong=Songs.ID AND ArtistsSongs.IDArtist="&id&" AND ArtistsSongs.PersonType=1 AND Songs.OrigYear!='-1' "&FilterSQL&"ORDER BY Songs.OrigYear"
      If Debug Then
        Call out("#"&sql)
      End If            
      Set iter2 = SDB.Database.OpenSQL(sql)
      If Not iter2.EOF Then
        orig1 = iter2.StringByIndex(0)
        iter2.Next
        While Not iter2.EOF
          orig1 = orig1&", "&iter2.StringByIndex(0)
          iter2.Next
        WEnd
        orig1 = " (Original: "&MapXML(FixYears(orig1))&")"
      End If      
      fout.WriteLine "    <h3>Year: "&MapXML(FixYears(href))&orig1&"</h3>"
    End If    
    
    'show links
    If (ShowLinks) And (artlnk.Count > 0) Then
      fout.WriteLine "    <h3>Links: "&BuildLinks(0,artlnk,id,name)&"</h3>"
    End If        

    'select albums by
    fout.WriteLine "    <table title=""Albums by"">"
    fout.WriteLine "      <tr>"        
    Dim name2,id2,year,year2
    Dim dic : Set dic = CreateObject("Scripting.Dictionary")
    If Not (Progress.Terminate) Then
      If Not (IncludeMode = 2) Then
        sql = "SELECT DISTINCT Albums.Album, Albums.ID"
        If AlbumsByYear Then 
          sql = sql&",CAST(Avg(Songs.Year/10000) AS INTEGER) AS AvgYear"
        End If        
        sql = sql&" FROM Albums,ArtistsAlbums,Songs WHERE ArtistsAlbums.IDArtist="&id&" AND ArtistsAlbums.IDAlbum=Albums.ID AND Songs.IDAlbum=ArtistsAlbums.IDAlbum "&FilterSQL&"GROUP BY Albums.Album, Albums.ID ORDER BY Albums.Album"
        If Debug Then
          Call out("#"&sql)
        End If        
        Set iter2 = SDB.Database.OpenSQL(sql)
        Do While Not iter2.EOF
          SDB.ProcessMessages
          If Progress.Terminate Then 
            Exit Do
          End If
          name2 = iter2.StringByIndex(0)
          id2 = iter2.StringByIndex(1)
          If (id2 = 0) Or (name2 = "") Then 
            name2 = UnknownAlbum
          End If
          year2 = "0000"
          If AlbumsByYear Then
            year2 = iter2.StringByIndex(2)
            If Len(year2) > 4 Then
              year2 = Left(year2,4)
            Else
              While Len(year2) < 4
                year2 = "0"&year2
              WEnd            
            End If
          End If          
          If IncludeMode = 3 Then
            dic.Item("1|"&year2&"|"&name2) = id2
          Else
            dic.Item(year2&"|"&name2) = id2
          End If
          iter2.Next
        Loop
      End If
    End If          
    
    'select albums featuring
    If Not (IncludeMode = 1) Then
      sql = "SELECT DISTINCT Albums.Album,Albums.ID"
      If AlbumsByYear Then 
        sql = sql&",CAST(Avg(Songs.Year/10000) AS INTEGER) AS AvgYear "
      End If
      sql = sql&" FROM Albums,Songs,ArtistsSongs WHERE Albums.ID=Songs.IDAlbum AND Albums.Album!='' AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.IDArtist="&id&" AND ArtistsSongs.PersonType=1 "&FilterSQL&"GROUP BY Albums.Album, Albums.ID"
      If Debug Then
        Call out("#"&sql)
      End If      
      Set iter2 = SDB.Database.OpenSQL(sql)
      Do While Not iter2.EOF
        SDB.ProcessMessages
        If Progress.Terminate Then 
          Exit Do
        End If
        name2 = iter2.StringByIndex(0)
        id2 = iter2.StringByIndex(1)
        If (id2 = 0) Or (name2 = "") Then 
          name2 = UnknownAlbum
        End If
        year2 = "0000"
        If AlbumsByYear Then
          year2 = iter2.StringByIndex(2)
          If Len(year2) > 4 Then
            year2 = Left(year2,4)
          Else
            While Len(year2) < 4
              year2 = "0"&year2
            WEnd          
          End If
        End If        
        If IncludeMode = 3 Then
          If Not (dic.Exists("1|"&year2&"|"&name2)) Then
            dic.Item("2|"&year2&"|"&name2) = id2
          End If
        Else          
          dic.Item(year2&"|"&name2) = id2
        End If
        iter2.Next
      Loop
    End If
    
    'sort albums
    Dim arr : arr = dic.Keys
    If Not (Progress.Terminate) Then
      Call SortArray(arr)
    End If
        
    'display albums
    j = 1
    Dim dit : Set dit = Nothing
    For i = 0 To UBound(arr)
      SDB.ProcessMessages
      If Progress.Terminate Then 
        Exit For
      End If
      If IncludeMode = 3 Then
        name2 = Mid(arr(i),8)
      Else    
        name2 = Mid(arr(i),6)
      End If
      id2 = dic.Item(arr(i))
      href = "#album_"&id2
      If SeparateFiles Then 
        href = "../albums/album_"&id2&ext
      End If
      If FramedView Then 
        href = "../albums/album_"&id2&ext&""" target=""main"
      End If
      cmp = ""
      If Not (CompAlbumMask = "") Or Not (CompTrackMask = "") Then
        sql = "SELECT Count(DISTINCT CASE WHEN InStr(Artist,';')>0 THEN SubStr(Artist,1,InStr(Artist,';')-1) WHEN InStr(Artist,';')=0 THEN Artist END) COLLATE IUNICODE FROM Songs WHERE IDAlbum="&id2
        If Debug Then
          Call out("#"&sql)
        End If      
        Set dit = SDB.Database.OpenSQL(sql)
        If dit.ValueByIndex(0) > 1 Then
          cmp = "Y"
        End If        
        comp.Item("#"&id2) = cmp
      End If
      If ApplyMask Then 
        name2 = GetAlbum(id2,0,ext,cmp)
      Else
        name2 = MapXML(name2)
      End If
    
      Dim tds : tds = "</td>"
      fout.WriteLine "        <td class=""album"" style=""vertical-align:bottom;"">"
      If DisplayTop = 0 Then
        If ShowTracks Then
          fout.WriteLine "        <p><a href="""&href&""">"&name2&"</a></p>"
        Else
          fout.WriteLine "        <p>"&name2&"</p>"
        End If
      Else
        If ShowTracks Then
          tds = "<p><a href="""&href&""">"&name2&"</a></p></td>"
        Else
          tds = "<p>"&name2&"</p></td>"
        End If
      End If
      If AlbumArtMode < 3 Then
        If AlbumArtIcon > 0 Then
          If ShowTracks Then
            fout.WriteLine "        <a href="""&href&"""><img src="""&GetAlbumArt(id2)&""" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image""></a>"&tds
          Else
            fout.WriteLine "        <img src="""&GetAlbumArt(id2)&""" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image"">"&tds
          End If
          j = j + 1
          If (AlbumsPerRow = 0 And j > ScreenWidth/(AlbumArtIcon+25)) Or (AlbumsPerRow > 0 And j > AlbumsPerRow) Then
            fout.WriteLine "      </tr>"
            fout.WriteLine "      <tr>"
            j = 1
          End If  
        Else
          fout.WriteLine "        "&tds
          fout.WriteLine "      </tr>"
          fout.WriteLine "      <tr>"
        End If
      Else
        fout.WriteLine "        "&tds
        fout.WriteLine "      </tr>"
        fout.WriteLine "      <tr>"
      End If
    Next
    If (ShowUnknown) And Not (Progress.Terminate) Then
      sql = "SELECT Count(*) FROM Songs,ArtistsSongs WHERE Songs.Album='' AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.IDArtist="&id&" "&FilterSQL
      If Debug Then
        Call out("#"&sql)
      End If    
      Set dit = SDB.Database.OpenSQL(sql)
      If dit.ValueByIndex(0) > 0 Then
        name2 = UnknownAlbum
        id2 = id&"u"
        href = "#album_"&id2
        If SeparateFiles Then 
          href = "../albums/album_"&id2&ext
        End If
        If FramedView Then 
          href = "../albums/album_"&id2&ext&""" target=""main"
        End If
        fout.WriteLine "        <td class=""album"" style=""vertical-align:bottom;"">"
        If DisplayTop = 0 Then
          If ShowTracks Then
            fout.WriteLine "        <p><a href="""&href&""">"&name2&"</a></p>"
          Else
            fout.WriteLine "        <p>"&name2&"</p>"
          End If
          tds = "</td>"
        Else
          If ShowTracks Then
            tds = "<p><a href="""&href&""">"&name2&"</a></p></td>"
          Else
            tds = "<p>"&name2&"</p></td>"
          End If
        End If
        If AlbumArtMode < 3 Then
          If AlbumArtIcon > 0 Then
            If ShowTracks Then
              fout.WriteLine "        <a href="""&href&"""><img src="""&ImageBase&"none.jpg"" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image""></a>"
            Else
              fout.WriteLine "        <img src="""&ImageBase&"none.jpg"" style=""width:"&AlbumArtIcon&"px;border:0px;"" alt=""Image"">"
            End If
          End If
        End If
        fout.WriteLine "        "&tds
      End If
    End If
    
    fout.WriteLine "        <td>&nbsp;</td>"
    fout.WriteLine "      </tr>"
    fout.WriteLine "    </table>"
    fout.WriteLine "    <br>"
    fout.WriteLine "  </div>"
    If SeparateFiles Or FramedView Then
      fout.WriteLine "</body>"
      fout.WriteLine "</html>"
      fout.Close
    End If
    iter.Next
  Loop
  
  If ShowTracks Then
    'build table headers
    Dim rowhead : rowhead = ""
    Dim cmphead : cmphead = ""
    If TableHeader Then
      If InStr(TrackMask,"<|>") > 0 Then
        rowhead = BuildHeader(TrackMask)
        If CompTrackMask = "" Then
          cmphead = rowhead
        Else
          cmphead = BuildHeader(CompTrackMask)
        End If                       
      Else
        If InStr(CompTrackMask,"<|>") > 0 Then
          cmphead = BuildHeader(CompTrackMask)
        End If
      End If
    End If  

    'loop through albums
    Dim table : table = False
    Dim tmp : tmp = -1
    sql = "SELECT DISTINCT Albums.Album,Albums.ID FROM Albums"
    If FilterSQL = "" Then
      FromClause1  = ""
      WhereClause1 = "Albums.Album!='' "
    Else
      FromClause1  = ", Songs"
      WhereClause1 = "Albums.Album!='' AND Songs.IDAlbum=Albums.ID "&FilterSQL
    End If
    sql = sql&FromClause1&" WHERE "&WhereClause1&"ORDER BY Albums.Album"
    If Debug Then
      Call out("#"&sql)
    End If      
    Set iter = SDB.Database.OpenSQL(sql)
    Do While Not iter.EOF
      SDB.ProcessMessages
      If Progress.Terminate Then 
        Exit Do
      End If
      name = iter.StringByName("Album")
      id = iter.StringByName("ID")
      If (id = 0) Or (name = "") Then 
        name = UnknownAlbum
      End If
      Progress.Increase
      UpdateProgress("Album: "&name)
    
      If SeparateFiles Or FramedView  Then
        href = AlbsDir&"\album_"&id&ext
        Set fout = fso.CreateTextFile(href,True,SupportUnicode) 
        fout.WriteLine "<!DOCTYPE html>"
        fout.WriteLine "<html>" 
        fout.WriteLine "<head>"
        fout.WriteLine "  <title>"&MapXML(name)&"</title>" 
        fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">" 
        fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
        fout.WriteLine "  <link rel=""stylesheet"" href=""../css/report.css"" type=""text/css"">" 
        fout.WriteLine "</head>"
        fout.WriteLine "<body>" 
        If SeparateFiles Then
          fout.WriteLine "  <h1>"
          fout.WriteLine "    <a href=""../"&fso.GetFileName(path)&""">"&PageTitle&"</a>"
          fout.WriteLine "  </h1>"
          fout.WriteLine "  <p>"
          fout.WriteLine "    Created: "&Date&" "&Time&" using '"&AppName&"' by <a href=""http://trixmoto.net"" target=""_blank"">Trixmoto</a>"
          fout.WriteLine "  </p>"
          fout.WriteLine "  <br>"
        End If
      End If    
    
      'check if table or not
      table = False
      cmp = comp.Item("#"&id)
      If (cmp = "Y") And Not (CompTrackMask = "") Then
        If InStr(CompTrackMask,"<|>") > 0 Then
          table = True
        End If
      Else
        If InStr(TrackMask,"<|>") > 0 Then
          table = True
        End If
      End If
          
      'loop through tracks
      sql = "AND Songs.IDAlbum="&id&" "&FilterSQL&"ORDER BY DiscNumber COLLATE NUMERICSTRING,TrackNumber COLLATE NUMERICSTRING"
      If Debug Then
        Call out("#"&sql)
      End If        
      Set iter2 = SDB.Database.QuerySongs(sql)
      If Not iter2.EOF Then
        fout.WriteLine "  <div class=""album_wrapper"">"
        fout.WriteLine "    <h2>"&GetAlbum(id,1,ext,cmp)&toplink&"</h2>"
        If Not (DetailsMask = "") Then
          fout.WriteLine "      "&GetAlbum(id,2,ext,"")
        End If
      
        'list genres
        Dim iter3 : Set iter3 = Nothing
        If ListGenres Then
          href = ""
          sql = "SELECT DISTINCT Genres.GenreName FROM Genres,Songs,GenresSongs WHERE GenresSongs.IDSong=Songs.ID AND GenresSongs.IDGenre=Genres.IDGenre AND Songs.IDAlbum="&id&" "&FilterSQL&"ORDER BY Genres.GenreName"
          If Debug Then
            Call out("#"&sql)
          End If            
          Set iter3 = SDB.Database.OpenSQL(sql)
          If Not iter3.EOF Then
            href = iter3.StringByName("GenreName")
            iter3.Next
            While Not iter3.EOF
              href = href&", "&iter3.StringByName("GenreName")
              iter3.Next
            WEnd
          End If
          fout.WriteLine "    <h3>Genre: "&MapXML(href)&"</h3>"        
        End If      
      
        'list years
        If ListYears Then
          href = ""
          sql = "SELECT DISTINCT Substr(Songs.Year,1,4) FROM Songs WHERE Songs.IDAlbum="&id&" "&FilterSQL&"ORDER BY Songs.Year"
          If Debug Then
            Call out("#"&sql)
          End If            
          Set iter3 = SDB.Database.OpenSQL(sql)
          If Not iter3.EOF Then
            href = iter3.StringByIndex(0)
            iter3.Next
            While Not iter3.EOF
              href = href&", "&iter3.StringByIndex(0)
              iter3.Next
            WEnd
          End If
          Dim orig2 : orig2 = ""
          sql = "SELECT DISTINCT Substr(Songs.OrigYear,1,4) FROM Songs WHERE Songs.IDAlbum="&id&" AND Songs.OrigYear!='-1' "&FilterSQL&"ORDER BY Songs.OrigYear"
          If Debug Then
            Call out("#"&sql)
          End If            
          Set iter3 = SDB.Database.OpenSQL(sql)
          If Not iter3.EOF Then
            orig2 = iter3.StringByIndex(0)
            iter3.Next
            While Not iter3.EOF
              orig2 = orig2&", "&iter3.StringByIndex(0)
              iter3.Next
            WEnd
            orig2 = " (Original: "&MapXML(FixYears(orig2))&")"
          End If        
          fout.WriteLine "    <h3>Year: "&MapXML(FixYears(href))&orig2&"</h3>"
        End If
      
        'show links
        If (ShowLinks) And (alblnk.Count > 0) Then
          fout.WriteLine "    <h3>Links: "&BuildLinks(1,alblnk,id,"")&"</h3>"
        End If        

        'track list header
        fout.WriteLine "    <div class=""tracks"">"
        If table Then
          fout.WriteLine "      <table title=""Track list"">"
          If TableHeader Then
            fout.WriteLine "        <tr class=""table_header"">"
            If cmp = "Y" Then
              fout.WriteLine "          <th>"&cmphead&"</th>"
            Else
              fout.WriteLine "          <th>"&rowhead&"</th>"
            End If
            fout.WriteLine "        </tr>"
          End If          
        Else
          fout.WriteLine "      <ul>"
        End If
      
        'list tracks (in each album)
        Err.Clear
        On Error Resume Next
        Do While Not iter2.EOF
          SDB.ProcessMessages
          If Progress.Terminate Then 
            Exit Do
          End If
          href = ""
          If Not (TooltipMask = "") Then
            href = " title="""&GetTooltip(iter2.Item)&""""
          End If
          If table Then
            fout.WriteLine "        <tr"&href&">"
            fout.WriteLine "          <td>"&GetTrack(iter2.Item,ext,cmp)&"</td>"
            fout.WriteLine "        </tr>"
          Else
            fout.WriteLine "        <li"&href&">"&GetTrack(iter2.Item,ext,cmp)&"</li>"
          End If        
          If Err.Number <> 0 Then
            If Debug Then
              Call out("***"&Err.Number&" - "&Err.Description&" ("&Err.Source&")")
            End If
            If TrackLinks > 0 Then                
              tmp = TrackLinks
              TrackLinks = 0
              If table Then
                fout.WriteLine "        <tr"&href&">"
                fout.WriteLine "          <td>"&GetTrack(iter2.Item,ext,cmp)&"</td>"
                fout.WriteLine "        </tr>"
              Else
                fout.WriteLine "        <li"&href&">"&GetTrack(iter2.Item,ext,cmp)&"</li>"
              End If           
              TrackLinks = tmp
            End If
            Err.Clear
          End If
          iter2.Next
        Loop  
        On Error Goto 0
        If table Then
          fout.WriteLine "      </table>"
        Else
          fout.WriteLine "      </ul>"
        End If      
        fout.WriteLine "    </div>"      
      
        'show artwork
        If AlbumArtMode < 3 Then
          If AlbumArtWidth > 0 Then
            fout.WriteLine "    <div class=""cover"">"
            fout.WriteLine "      <img src="""&GetAlbumArt(id)&""" style=""width:"&AlbumArtWidth&"px;border:0px;"" alt=""Image"">"
            fout.WriteLine "    </div>"
          End If
        End If
        fout.WriteLine "  </div>"
        fout.WriteLine "  <br>"
      End If
    
      If SeparateFiles Or FramedView  Then
        fout.WriteLine "</body>"
        fout.WriteLine "</html>"
        fout.Close 
      End If    
    
      Progress.Increase
      iter.Next
    Loop
  End If
  
  If ShowTracks And ShowUnknown And Not Progress.Terminate Then
    sql = "SELECT ArtistsSongs.IDArtist FROM Songs,ArtistsSongs WHERE Songs.Album='' AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.PersonType=1 "&FilterSQL&"GROUP BY ArtistsSongs.IDArtist"
    If Debug Then
      Call out("#"&sql)
    End If        
    Set iter = SDB.Database.OpenSQL(sql)
    Do While Not iter.EOF
      SDB.ProcessMessages
      If Progress.Terminate Then 
        Exit Do
      End If
      name = UnknownAlbum
      id = iter.StringByIndex(0)&"u"
      Progress.Increase
      UpdateProgress("Album: "&name)
    
      If SeparateFiles Or FramedView  Then
        href = AlbsDir&"\album_"&id&ext
        Set fout = fso.CreateTextFile(href,True,SupportUnicode) 
        fout.WriteLine "<!DOCTYPE html>"
        fout.WriteLine "<html>" 
        fout.WriteLine "<head>"
        fout.WriteLine "  <title>"&MapXML(name)&"</title>" 
        fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">" 
        fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
        fout.WriteLine "  <link rel=""stylesheet"" href=""../css/report.css"" type=""text/css"">" 
        fout.WriteLine "</head>"
        fout.WriteLine "<body>" 
        If SeparateFiles Then
          fout.WriteLine "  <h1>"
          fout.WriteLine "    <a href=""../"&fso.GetFileName(path)&""">"&PageTitle&"</a>"
          fout.WriteLine "  </h1>"
          fout.WriteLine "  <p>"
          fout.WriteLine "    Created: "&Date&" "&Time&" using '"&AppName&"' by <a href=""http://trixmoto.net"" target=""_blank"">Trixmoto</a>"
          fout.WriteLine "  </p>"
          fout.WriteLine "  <br>"
        End If
      End If    
    
      'check if table or not
      table = False
      If InStr(TrackMask,"<|>") > 0 Then
        table = True
      End If
    
      Dim sqlin : sqlin = "SELECT Songs.ID FROM Songs,ArtistsSongs WHERE Songs.Album='' AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.PersonType=1 AND ArtistsSongs.IDArtist="&iter.StringByIndex(0)&" "&FilterSQL
      sql = "AND Songs.ID IN ("&sqlin&")"
      If Debug Then
        Call out("#"&sql)
      End If          
      Set iter2 = SDB.Database.QuerySongs(sql)
      If Not iter2.EOF Then
        fout.WriteLine "  <div class=""album_wrapper"">"
        fout.WriteLine "    <h2>"&GetUnknown(iter.StringByIndex(0),ext,sqlin)&toplink&"</h2>"
      
        'list genres
        Dim iter4 : Set iter4 = Nothing
        If ListGenres Then
          href = ""
          sql = "SELECT DISTINCT Genres.GenreName FROM Songs,ArtistsSongs,Genres,GenresSongs WHERE Songs.Album='' AND Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.PersonType=1 AND ArtistsSongs.IDArtist="&iter.StringByIndex(0)&" AND GenresSongs.IDGenre=Genres.IDGenre AND GenresSongs.IDSong=Songs.ID AND ArtistsSongs.PersonType=1 "&FilterSQL&"ORDER BY Genres.GenreName"
          If Debug Then
            Call out("#"&sql)
          End If              
          Set iter4 = SDB.Database.OpenSQL(sql)
          If Not iter4.EOF Then
            href = iter4.StringByName("GenreName")
            iter4.Next
            While Not iter4.EOF
              href = href&", "&iter4.StringByName("GenreName")
              iter4.Next
            WEnd
          End If
          fout.WriteLine "    <h3>Genre: "&MapXML(href)&"</h3>"        
        End If      

        'list years
        If ListYears Then
          href = ""
          sql = "SELECT DISTINCT Substr(Songs.Year,1,4) FROM Songs WHERE Songs.ID IN ("&sqlin&") ORDER BY Songs.Year"
          If Debug Then
            Call out("#"&sql)
          End If              
          Set iter4 = SDB.Database.OpenSQL(sql)
          If Not iter4.EOF Then
            href = iter4.StringByIndex(0)
            iter4.Next
            While Not iter4.EOF
              href = href&", "&iter4.StringByIndex(0)
              iter4.Next
            WEnd
          End If
          Dim orig3 : orig3 = ""
          sql = "SELECT DISTINCT Substr(Songs.OrigYear,1,4) FROM Songs WHERE Songs.ID IN ("&sqlin&") AND Songs.OrigYear!='-1' ORDER BY Songs.OrigYear"
          If Debug Then
            Call out("#"&sql)
          End If              
          Set iter4 = SDB.Database.OpenSQL(sql)
          If Not iter4.EOF Then
            orig3 = iter4.StringByIndex(0)
            iter4.Next
            While Not iter4.EOF
              orig3 = orig3&", "&iter4.StringByIndex(0)
              iter4.Next
            WEnd
            orig3 = " (Original: "&MapXML(FixYears(orig3))&")"
          End If          
          fout.WriteLine "    <h3>Year: "&MapXML(FixYears(href))&orig3&"</h3>"
        End If                

        'list tracks (in unknown album)
        fout.WriteLine "    <div class=""tracks"">"
        If table Then
          fout.WriteLine "      <table title=""Track list"">"
          If TableHeader Then
            fout.WriteLine "        <tr class=""table_header"">"
            fout.WriteLine "          <th>"&rowhead&"</th>"
            fout.WriteLine "        </tr>"
          End If
        Else
          fout.WriteLine "      <ul>"
        End If
        Err.Clear
        On Error Resume Next
        Do While Not iter2.EOF
          SDB.ProcessMessages
          If Progress.Terminate Then 
            Exit Do
          End If
          href = ""
          If Not (TooltipMask = "") Then
            href = " title="""&GetTooltip(iter2.Item)&""""
          End If          
          If table Then
            fout.WriteLine "        <tr"&href&">"
            fout.WriteLine "          <td>"&GetTrack(iter2.Item,ext,"")&"</td>"
            fout.WriteLine "        </tr>"
          Else
            fout.WriteLine "        <li"&href&">"&GetTrack(iter2.Item,ext,"")&"</li>"
          End If        
          If Err.Number <> 0 Then
            If Debug Then
              Call out("***"&Err.Number&" - "&Err.Description&" ("&Err.Source&")")
            End If
            If TrackLinks > 0 Then          
              tmp = TrackLinks
              TrackLinks = 0
              If table Then
                fout.WriteLine "        <tr"&href&">"
                fout.WriteLine "          <td>"&GetTrack(iter2.Item,ext,"")&"</td>"
                fout.WriteLine "        </tr>"
              Else
                fout.WriteLine "        <li"&href&">"&GetTrack(iter2.Item,ext,"")&"</li>"
              End If           
              TrackLinks = tmp
            End If
            Err.Clear
          End If
          iter2.Next
        Loop  
        On Error Goto 0
        If table Then
          fout.WriteLine "      </table>"
        Else
          fout.WriteLine "      </ul>"
        End If      
        fout.WriteLine "    </div>"
      
        'show artwork
        If AlbumArtMode < 3 Then
          If AlbumArtWidth > 0 Then
            fout.WriteLine "    <div class=""cover"">"
            fout.WriteLine "      <img src="""&ImageBase&"none.jpg"" style=""width:"&AlbumArtWidth&"px;border:0px;"" alt=""Image"">"
            fout.WriteLine "    </div>"
          End If
        End If
        fout.WriteLine "  </div>"
        fout.WriteLine "  <br>"
      End If
    
      If SeparateFiles Or FramedView  Then
        fout.WriteLine "</body>"
        fout.WriteLine "</html>"
        fout.Close 
      End If    
      iter.Next
    Loop
  End If
  
  If Playlists Then
    Progress.Value = Progress.Value+3
    UpdateProgress("Playlist: (Root)")
    SDB.ProcessMessages
    
    If SeparateFiles Or FramedView Then
      href = PlayDir&"\play"&ext
      Set fout = fso.CreateTextFile(href,True,SupportUnicode) 
      fout.WriteLine "<!DOCTYPE html>"
      fout.WriteLine "<html>" 
      fout.WriteLine "<head>"
      fout.WriteLine "  <title>Playlists</title>" 
      fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">" 
      fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
      fout.WriteLine "  <link rel=""stylesheet"" href=""../css/report.css"" type=""text/css"">" 
      fout.WriteLine "</head>"
      fout.WriteLine "<body>" 
      If SeparateFiles Then
        fout.WriteLine "  <h1>"
        fout.WriteLine "    <a href=""../"&fso.GetFileName(path)&""">"&PageTitle&"</a>"
        fout.WriteLine "  </h1>"
        fout.WriteLine "  <p>"
        fout.WriteLine "    Created: "&Date&" "&Time&" using '"&AppName&"' by <a href=""http://trixmoto.net"" target=""_blank"">Trixmoto</a>"
        fout.WriteLine "  </p>"
        fout.WriteLine "  <br>"
      End If
    End If    
    fout.WriteLine "  <div class=""album_wrapper"">"
    fout.WriteLine "    <h2><a id=""play"">Playlists</a>"&toplink&"</h2>"
    fout.WriteLine "    <div class=""tracks"">"              
    
    Dim num : num = 0
    Dim ply : Set ply = Nothing
    Dim lev : lev = 0    
    For num = 0 To plays-1
      SDB.ProcessMessages
      If Progress.Terminate Then 
        Exit For
      End If
      name = ""
      Set ply = Nothing
      lev = 0
      tmp = num+1
      Call GetPlaylist(tmp,name,ply,lev)     
      href = "#play_"&ply.ID
      If SeparateFiles Then 
        href = "play_"&ply.ID&ext
      End If
      If FramedView Then 
        href = "play_"&ply.ID&ext&""" target=""main"
      End If
      If (ply.isAutoPlaylist) Or (ply.Tracks.Count = 0) Or (ShowTracks = False) Then
        fout.WriteLine GetIndent(lev)&MapXML(name)&"<br>"
      Else
        fout.WriteLine GetIndent(lev)&"<a href="""&href&""">"&MapXML(name)&"</a><br>"
      End If
    Next
    fout.WriteLine "    </div>"
    fout.WriteLine "  </div>"
    If SeparateFiles Or FramedView Then    
      fout.WriteLine "</body>"
      fout.WriteLine "</html>"
      fout.Close
    End If 
    
    If ShowTracks = False Then
      plays = 0
    End If
    For num = 0 To plays-1
      SDB.ProcessMessages
      If Progress.Terminate Then 
        Exit For
      End If
      name = ""
      Set ply = Nothing
      lev = 0
      tmp = num+1
      Call GetPlaylist(tmp,name,ply,lev)
      id = ply.ID
      Progress.Value = Progress.Value+3
      UpdateProgress("Playlist: "&name)
    
      If Not(ply.isAutoPlaylist) And (ply.Tracks.Count > 0) Then
        If SeparateFiles Or FramedView Then
          href = PlayDir&"\play_"&id&ext
          Set fout = fso.CreateTextFile(href,True,SupportUnicode) 
          fout.WriteLine "<!DOCTYPE html>"
          fout.WriteLine "<html>" 
          fout.WriteLine "<head>"
          fout.WriteLine "  <title>"&MapXML(name)&"</title>" 
          fout.WriteLine "  <meta http-equiv=""Content-Type"" content=""text/html; charset=utf-8"">" 
          fout.WriteLine "  <meta http-equiv=""Content-Style-Type"" content=""text/css"">"
          fout.WriteLine "  <link rel=""stylesheet"" href=""../css/report.css"" type=""text/css"">" 
          fout.WriteLine "</head>"
          fout.WriteLine "<body>" 
          If SeparateFiles Then
            fout.WriteLine "  <h1>"
            fout.WriteLine "    <a href=""../"&fso.GetFileName(path)&""">"&PageTitle&"</a>"
            fout.WriteLine "  </h1>"
            fout.WriteLine "  <p>"
            fout.WriteLine "    Created: "&Date&" "&Time&" using '"&AppName&"' by <a href=""http://trixmoto.net"" target=""_blank"">Trixmoto</a>"
            fout.WriteLine "  </p>"
            fout.WriteLine "  <br>"
          End If
        End If    
        href = "#play"
        If SeparateFiles Then 
          href = "play"&ext
        End If
        If FramedView Then 
          href = "play"&ext&""" target=""main"
        End If  
        fout.WriteLine "  <div class=""album_wrapper"">"
        fout.WriteLine "    <h2><a id=""play_"&id&""" href="""&href&""">"&MapXML(name)&"</a>"&toplink&"</h2>"            
        
        table = False
        If Not (CompTrackMask = "") Then
          If InStr(CompTrackMask,"<|>") > 0 Then
            table = True
          End If
        Else
          If InStr(TrackMask,"<|>") > 0 Then
            table = True
          End If
        End If              
  
        'list tracks (in playlist)
        fout.WriteLine "    <div class=""tracks"">"        
        Dim lst : Set lst = ply.Tracks
        If table Then
          fout.WriteLine "      <table title=""Track list"">"
          If TableHeader Then
            fout.WriteLine "        <tr class=""table_header"">"
            fout.WriteLine "          <th>"&cmphead&"</th>"
            fout.WriteLine "        </tr>"
          End If
        Else
          fout.WriteLine "      <ul>"
        End If
        Err.Clear
        On Error Resume Next          
        For j = 0 To lst.Count-1
          SDB.ProcessMessages
          If Progress.Terminate Then 
            Exit For
          End If
          Dim itm : Set itm = lst.Item(j)
          href = ""
          If Not (TooltipMask = "") Then
            href = " title="""&GetTooltip(itm)&""""
          End If          
          If table Then
            fout.WriteLine "        <tr"&href&">"
            fout.WriteLine "          <td>"&GetTrack(itm,ext,"Y")&"</td>"
            fout.WriteLine "        </tr>"
          Else
            fout.WriteLine "        <li"&href&">"&GetTrack(itm,ext,"Y")&"</li>"
          End If        
          If Err.Number <> 0 Then
            If Debug Then
              Call out("***"&Err.Number&" - "&Err.Description&" ("&Err.Source&")")
            End If
            If TrackLinks > 0 Then          
              tmp = TrackLinks
              TrackLinks = 0
              If table Then
                fout.WriteLine "        <tr"&href&">"
                fout.WriteLine "          <td>"&GetTrack(itm,ext,"Y")&"</td>"
                fout.WriteLine "        </tr>"
              Else
                fout.WriteLine "        <li"&href&">"&GetTrack(itm,ext,"Y")&"</li>"
              End If           
              TrackLinks = tmp
            End If
            Err.Clear
          End If
        Next
        On Error Goto 0
        If table Then
          fout.WriteLine "      </table>"
        Else
          fout.WriteLine "      </ul>"
        End If      
        fout.WriteLine "    </div>"           
        fout.WriteLine "  </div>"
        fout.WriteLine "  <br>"    
        If SeparateFiles Or FramedView  Then
          fout.WriteLine "</body>"
          fout.WriteLine "</html>"
          fout.Close
        End If    
      End If
    Next
  End If  
 
  If Not (SeparateFiles Or FramedView) Then
    fout.WriteLine "</body>"
    fout.WriteLine "</html>"
    fout.Close 
  End If
  
  If Progress.Terminate Then
    fso.DeleteFile(path)
    fso.DeleteFolder(Left(ImgsDir,Len(ImgsDir)-1))
    fso.DeleteFolder(Left(StylDir,Len(StylDir)-1))
    If SeparateFiles Or FramedView Then
      fso.DeleteFolder(Left(ArtsDir,Len(ArtsDir)-1))
      If ShowTracks Then
        fso.DeleteFolder(Left(AlbsDir,Len(AlbsDir)-1))
      End If
      If Playlists Then
        fso.DeleteFolder(Left(PlayDir,Len(PlayDir)-1))
      End If
    End If
    If FramedView Then
      href = BaseDir&"\f_caps"&ext
      If fso.FileExists(href) Then fso.DeleteFile(href)
      href = BaseDir&"\f_list"&ext
      If fso.FileExists(href) Then fso.DeleteFile(href)
      If ShowStats Then
        href = BaseDir&"\f_stat"&ext
        If fso.FileExists(href) Then fso.DeleteFile(href)
      End If      
    End If
    sql = UpdateProgress("terminated by user, output files deleted")
    i = SDB.MessageBox(sql,mtInformation,Array(mbOk))
  Else 
    sql = UpdateProgress("created in "&GetTime(Timer-starttime))
    i = SDB.MessageBox(sql&". Display now?",mtConfirmation,Array(mbYes,mbNo))    
    If i = mrYes Then
      Dim WShell : Set WShell = CreateObject("WScript.Shell")
      i = WShell.Run(Chr(34)&path&Chr(34),1,0)
      Set WShell = Nothing
    End If
  End If
  Set Progress = Nothing
End Sub 

Sub BrowseClick(ClickedBtn)
  'initialise
  Dim dlg : Set dlg = SDB.CommonDialog
  Dim edt : Set edt = ClickedBtn.Common.TopParent.Common.ChildControl("Path")
  'show dialog
  dlg.DefaultExt = SDB.IniFile.StringValue("AdvancedReport","Extension")
  dlg.Filter = "HTML (*.htm)|*.htm|HTML (*.html)|*.html|PHP (*.php)|*.php"
  dlg.Flags = cdlOFNOverwritePrompt + cdlOFNHideReadOnly + cdlOFNNoChangeDir
  dlg.InitDir = edt.Text
  dlg.ShowSave
  'check selection
  If Not dlg.Ok Then Exit Sub
  If dlg.FileName = "" Then Exit Sub
  'update setting
  edt.Text = dlg.FileName
End Sub

Sub EditClick(ClickedBtn)
  Dim wsh : Set wsh = CreateObject("WScript.Shell")
  Dim temp : temp = SDB.IniFile.StringValue("AdvancedReport","Path")
  temp = fso.GetParentFolderName(temp)&"\css\"
  Dim css : css = temp&"report.css"
  'copy default file if not there  
  If Not (fso.FileExists(css)) Then
    If fso.FileExists(SDB.ScriptsPath&"report.css") Then
      Call GeneratePath(temp)
      Call fso.CopyFile(SDB.ScriptsPath&"report.css",temp,True)
    End If      
  End If
  'open file in notepad
  Call wsh.Run("%windir%\system32\notepad.exe "&Chr(34)&css&Chr(34),1,0)
End Sub

Sub IconsClick(ClickedBtn)
  ShowIcons = SDB.IniFile.StringValue("AdvancedReport","ShowIcons")
  Dim arr : arr = Split(ShowIcons,";")
  Dim dic : Set dic = CreateObject("Scripting.Dictionary")
  Dim i : i = 0
  For i = 0 To UBound(arr)
    dic.Item(Left(arr(i),1)) = Mid(arr(i),2,1)
  Next  
  Dim Form : Set Form = SDB.UI.NewForm
  Form.Common.SetRect 100, 100, 295, 410
  Form.BorderStyle  = 2   
  Form.FormPosition = 4   
  Form.Caption = AppName&" - Icons"
  Dim c : c = ""
  Dim l : l = 10
  Dim k : k = 10
  For i = 1 To 26
    If i > 13 Then
      l = 150
      k = 335
    End If
    c = Chr(64+i)
    Dim Label : Set Label = SDB.UI.NewLabel(Form)
    Label.Caption = c&":"
    Label.Common.Left = l
    Label.Common.Top = (25*i)-k
    Dim Drop : Set Drop = SDB.UI.NewDropDown(Form)
    Drop.Common.ControlName = "Drop"&c
    Drop.Common.Left = l+20
    Drop.Common.Top = Label.Common.Top -4
    Drop.Common.Width = 100
    Drop.AddItem("None")
    Drop.AddItem("CD / DVD")
    Drop.AddItem("Hard disk")
    Drop.AddItem("IPod")
    Drop.AddItem("Server")
    Drop.AddItem("USB pen")
    Drop.Style = 2
    If dic.Exists(c) Then
      Drop.ItemIndex = Int(dic.Item(c))
    Else
      Drop.ItemIndex = 0
    End If
  Next  
  Dim BtnOk : Set BtnOk = SDB.UI.NewButton(Form)
  BtnOk.Caption = "&Ok"
  BtnOk.Common.Top = Form.Common.Height - BtnOk.Common.Height -35
  BtnOk.Common.Left = Form.Common.Width - BtnOk.Common.Width -25
  BtnOk.UseScript = Script.ScriptPath
  BtnOk.Default = True
  BtnOk.ModalResult = 1   
  If Not (Form.ShowModal = 1) Then 
    Exit Sub
  End If
  ShowIcons = ""
  For i = 1 To 26
    c = Chr(64+i)
    Dim Edit : Set Edit = Form.Common.ChildControl("Drop"&c)
    ShowIcons = ShowIcons&";"&c&Edit.ItemIndex
  Next      
  SDB.IniFile.StringValue("AdvancedReport","ShowIcons") = Mid(ShowIcons,2)
End Sub

Function MapXML(srcstring)
  MapXML = srcstring
  If MapXML = "" Then
    Exit Function
  End If
  If Not SupportUnicode Then
    MapXML = SDB.ToAscii(MapXML)
  End If
  MapXML = Replace(MapXML,"&","&amp;")
  MapXML = Replace(MapXML,"<","&lt;")
  MapXML = Replace(MapXML,">","&gt;")
  Dim i : i = 1
  While i <= Len(MapXML)
    If AscW(Mid(MapXML,i,1)) > 127 Then
      MapXML = Mid(MapXML,1,i-1)+"&#"+CStr(AscW(Mid(MapXML,i,1)))+";"+Mid(MapXML,i+1,Len(MapXML))
    End If
    i = i+1
  WEnd
End Function

Function GetTrack(itm,ext,cmp)
  Dim temp,track,title,artist,album,albumartist,genre,theyear,thelink,artlink,ttllink,icon
  Dim bitrate,bpm,composer,cust1,cust2,cust3,path,length,rating,stars,bombs,href,sql,comment
  Dim band,chan,cond,copy,cust4,cust5,disc,group,lyri,mood,occa,mcomp,publ,qual,tempo,enco
  Dim thedate,origyear,lyrics,starttime,stoptime,skipcount,tracktype,series,director,producer
  Dim actors,parentalrating,episode,season  
  
  track = CStr(itm.TrackOrder)
  Do While Len(track)<2
    track = "0"&track
  Loop
  title = MapXML(itm.Title)
  artist = MapXML(itm.ArtistName)
  album = MapXML(itm.AlbumName)
  albumartist = MapXML(itm.AlbumArtistName)
  genre = MapXML(itm.Genre)
  theyear = Left(itm.Year,4)
  bitrate = Int(itm.Bitrate/1000)&"kbps"
  bpm = itm.BPM
  composer = MapXML(itm.Author)
  cust1 = MapXML(itm.Custom1)
  cust2 = MapXML(itm.Custom2)
  cust3 = MapXML(itm.Custom3)
  path = MapXML(itm.Path)
  length = GetTime(itm.SongLength/1000)
  temp = itm.Rating
  stars = GetStars(temp)
  bombs = GetBombs(temp)
  If temp < 0 Then 
    rating = "?"
  Else
    temp = temp / 20
    If temp = Int(temp) Then
      rating = temp
    Else
      rating = FormatNumber(temp,1)
    End If
  End If
  band = MapXML(itm.Band)
  chan = MapXML(itm.Channels)
  cond = MapXML(itm.Conductor)
  copy = MapXML(itm.Copyright)
  lyri = MapXML(itm.Lyricist)
  mood = MapXML(itm.Mood)
  mcomp = MapXML(itm.MusicComposer)
  occa = MapXML(itm.Occasion)
  publ = MapXML(itm.Publisher)
  qual = MapXML(itm.Quality)
  tempo = MapXML(itm.Tempo)
  enco = MapXML(itm.Encoder)
  If TrackLinks = 2 Then
    thelink = GetLink(GetRelative(itm.Path,AlbsDir))
  Else
    thelink = GetLink(itm.Path)
  End If
  ttllink = "<a href="""&thelink&""">"&title&"</a>"
  artlink = ""
  sql = "SELECT IDArtist,Artist FROM ArtistsSongs,Artists WHERE ArtistsSongs.IDArtist=Artists.ID AND ArtistsSongs.PersonType=1 AND ArtistsSongs.IDSong="&itm.ID
  If Debug Then
    Call out("#"&sql)
  End If      
  Dim dit : Set dit = SDB.Database.OpenSQL(sql)
  While Not (dit.EOF)
    href = "#artist_"&dit.ValueByIndex(0)
    If SeparateFiles Then 
      href = "../artists/artist_"&dit.ValueByIndex(0)&ext
    End If
    If FramedView Then 
      href = "../artists/artist_"&dit.ValueByIndex(0)&ext&""" target=""main"
    End If
    artlink = artlink&Delimiter&"<a href="""&href&""">"&MapXML(dit.StringByIndex(1))&"</a>"        
    dit.Next
  WEnd
  If Len(artlink) > 0 Then
    artlink = Mid(artlink,3)
  End If
  cust4 = MapXML(itm.Custom4)
  cust5 = MapXML(itm.Custom5)
  disc = MapXML(itm.DiscNumberStr)
  If disc = "" Then 
    disc = "1" 
  End If
  group = MapXML(itm.Grouping)
  comment = MapXML(itm.Comment)
  thedate = MapXML(itm.Date)
  If itm.OriginalYear = "" Or itm.OriginalYear = "-1" Then
    origyear = theyear
  Else
    origyear = Left(itm.OriginalYear,4)
  End If
  lyrics = MapXML(itm.Lyrics)
  starttime = GetTime(itm.StartTime/1000)
  stoptime = GetTime(itm.StopTime/1000)
  skipcount = MapXML(itm.SkipCount)
  tracktype = MapXML(itm.TrackType)
  series = MapXML(itm.Series)
  director = MapXML(itm.Director)
  producer = MapXML(itm.Producer)
  actors = MapXML(itm.Actors)
  parentalrating = MapXML(itm.ParentalRating)
  episode = MapXML(itm.EpisodeNumber)
  season = MapXML(itm.SeasonNumber)
  
  If (cmp = "Y") And Not (CompTrackMask = "") Then
    GetTrack = CompTrackMask
  Else
    GetTrack = TrackMask
  End If
  GetTrack = Replace(GetTrack,"<Track#>",track)
  GetTrack = Replace(GetTrack,"<Artist>",artist)
  GetTrack = Replace(GetTrack,"<Title>",title)
  GetTrack = Replace(GetTrack,"<Album>",album)
  GetTrack = Replace(GetTrack,"<Album Artist>",albumartist)
  GetTrack = Replace(GetTrack,"<AlbumArtist>",albumartist)
  GetTrack = Replace(GetTrack,"<Genre>",genre)
  GetTrack = Replace(GetTrack,"<Year>",theyear)
  GetTrack = Replace(GetTrack,"<Bitrate>",bitrate)
  GetTrack = Replace(GetTrack,"<BPM>",bpm)
  GetTrack = Replace(GetTrack,"<Composer>",composer)
  GetTrack = Replace(GetTrack,"<Custom 1>",cust1)
  GetTrack = Replace(GetTrack,"<Custom 2>",cust2)
  GetTrack = Replace(GetTrack,"<Custom 3>",cust3)
  GetTrack = Replace(GetTrack,"<Filename>",path)
  GetTrack = Replace(GetTrack,"<Length>",length)
  GetTrack = Replace(GetTrack,"<Rating>",rating)
  GetTrack = Replace(GetTrack,"<Stars>",stars)
  GetTrack = Replace(GetTrack,"<Bombs>",bombs)
  GetTrack = Replace(GetTrack,"<|>","</td><td>")
  GetTrack = Replace(GetTrack,"<Band>",band)
  GetTrack = Replace(GetTrack,"<Channels>",chan)
  GetTrack = Replace(GetTrack,"<Conductor>",cond)
  GetTrack = Replace(GetTrack,"<Copyright>",copy)
  GetTrack = Replace(GetTrack,"<Lyricist>",lyri)
  GetTrack = Replace(GetTrack,"<Mood>",mood)
  GetTrack = Replace(GetTrack,"<MusicComposer>",mcomp)
  GetTrack = Replace(GetTrack,"<Occasion>",occa)
  GetTrack = Replace(GetTrack,"<Publisher>",publ)
  GetTrack = Replace(GetTrack,"<Quality>",qual)
  GetTrack = Replace(GetTrack,"<Tempo>",tempo)
  GetTrack = Replace(GetTrack,"<Encoder>",enco)
  GetTrack = Replace(GetTrack,"<Link>","<a href="""&thelink&""">Link</a>")
  GetTrack = Replace(GetTrack,"<ArtistLink>",artlink)
  GetTrack = Replace(GetTrack,"<TitleLink>",ttllink)
  GetTrack = Replace(GetTrack,"<Custom 4>",cust4)
  GetTrack = Replace(GetTrack,"<Custom 5>",cust5)
  GetTrack = Replace(GetTrack,"<Disc#>",disc)
  GetTrack = Replace(GetTrack,"<Grouping>",group)
  GetTrack = Replace(GetTrack,"<Comment>",comment)
  GetTrack = Replace(GetTrack,"<Date>",thedate)
  GetTrack = Replace(GetTrack,"<OrigYear>",origyear)
  GetTrack = Replace(GetTrack,"<Lyrics>",lyrics)
  GetTrack = Replace(GetTrack,"<StartTime>",starttime)
  GetTrack = Replace(GetTrack,"<StopTime>",stoptime)
  GetTrack = Replace(GetTrack,"<SkipCount>",skipcount)
  GetTrack = Replace(GetTrack,"<TrackType>",tracktype)
  GetTrack = Replace(GetTrack,"<Series>",series)
  GetTrack = Replace(GetTrack,"<Director>",director)
  GetTrack = Replace(GetTrack,"<Producer>",producer)
  GetTrack = Replace(GetTrack,"<Actors>",actors)
  GetTrack = Replace(GetTrack,"<ParentalRating>",parentalrating)
  GetTrack = Replace(GetTrack,"<EpisodeNumber>",episode)
  GetTrack = Replace(GetTrack,"<SeasonNumber>",season)
  GetTrack = Replace(GetTrack,"<Custom1>",cust1)
  GetTrack = Replace(GetTrack,"<Custom2>",cust2)
  GetTrack = Replace(GetTrack,"<Custom3>",cust3)
  GetTrack = Replace(GetTrack,"<Custom4>",cust4)
  GetTrack = Replace(GetTrack,"<Custom5>",cust5)  
  
  If InStr(GetTrack,"<MediaIcon>") > 0 Then
    icon = ""
    temp = InStr(ShowIcons,Left(path,1))
    If temp > 0 Then
      temp = Int(Mid(ShowIcons,temp+1,1))
    End If
    Select Case temp
      Case 1 
        icon = "<img src="""&ImageBase&"cd-dvd.png"" alt=""CD/DVD"" border=""0"">"
      Case 2
        icon = "<img src="""&ImageBase&"harddisk.png"" alt=""Hard disk"" border=""0"">"
      Case 3
        icon = "<img src="""&ImageBase&"ipod.png"" alt=""iPod"" border=""0"">"
      Case 4
        icon = "<img src="""&ImageBase&"server.png"" alt=""Server"" border=""0"">"
      Case 5
        icon = "<img src="""&ImageBase&"usbpen.png"" alt=""USB pen"" border=""0"">"
    End Select  
    GetTrack = Replace(GetTrack,"<MediaIcon>",icon)  
  End If
  
  If TrackLinks > 0 Then 
    If InStr(GetTrack,"</a>") = 0 Then
      GetTrack = "<a href="""&thelink&""">"&GetTrack&"</a>"
    End If
  End If
End Function

Function GetTooltip(itm)
  Dim temp,track,title,artist,album,albumartist,genre,theyear,comment
  Dim bitrate,bpm,composer,cust1,cust2,cust3,path,length,rating,href
  Dim band,chan,cond,copy,cust4,cust5,disc,group,lyri,mood,occa,mcomp
  Dim publ,qual,tempo,enco,thedate,origyear,lyrics,starttime,stoptime
  Dim skipcount,tracktype,series,director,producer,actors,parentalrating
  Dim episode,season
  
  track = CStr(itm.TrackOrder)
  Do While Len(track)<2
    track = "0"&track
  Loop
  title = MapXML(itm.Title)
  artist = MapXML(itm.ArtistName)
  album = MapXML(itm.AlbumName)
  albumartist = MapXML(itm.AlbumArtistName)
  genre = MapXML(itm.Genre)
  theyear = Left(itm.Year,4)
  bitrate = Int(itm.Bitrate/1000)&"kbps"
  bpm = itm.BPM
  composer = MapXML(itm.Author)
  cust1 = MapXML(itm.Custom1)
  cust2 = MapXML(itm.Custom2)
  cust3 = MapXML(itm.Custom3)
  path = MapXML(itm.Path)
  length = GetTime(itm.SongLength/1000)
  temp = itm.Rating
  If temp < 0 Then 
    rating = "?"
  Else
    temp = temp / 20
    If temp = Int(temp) Then
      rating = temp
    Else
      rating = FormatNumber(temp,1)
    End If
  End If
  band = MapXML(itm.Band)
  chan = MapXML(itm.Channels)
  cond = MapXML(itm.Conductor)
  copy = MapXML(itm.Copyright)
  lyri = MapXML(itm.Lyricist)
  mood = MapXML(itm.Mood)
  mcomp = MapXML(itm.MusicComposer)
  occa = MapXML(itm.Occasion)
  publ = MapXML(itm.Publisher)
  qual = MapXML(itm.Quality)
  tempo = MapXML(itm.Tempo)
  enco = MapXML(itm.Encoder)
  cust4 = MapXML(itm.Custom4)
  cust5 = MapXML(itm.Custom5)
  disc = MapXML(itm.DiscNumberStr)
  group = MapXML(itm.Grouping)
  comment = MapXML(itm.Comment)
  thedate = MapXML(itm.Date)
  If itm.OriginalYear = "" Or itm.OriginalYear = "-1" Then
    origyear = theyear
  Else
    origyear = Left(itm.OriginalYear,4)
  End If
  lyrics = MapXML(itm.Lyrics)
  starttime = GetTime(itm.StartTime/1000)
  stoptime = GetTime(itm.StopTime/1000)
  skipcount = MapXML(itm.SkipCount)
  tracktype = MapXML(itm.TrackType)
  series = MapXML(itm.Series)
  director = MapXML(itm.Director)
  producer = MapXML(itm.Producer)
  actors = MapXML(itm.Actors)
  parentalrating = MapXML(itm.ParentalRating)
  episode = MapXML(itm.EpisodeNumber)
  season = MapXML(itm.SeasonNumber)  
 
  GetTooltip = TooltipMask
  GetTooltip = Replace(GetTooltip,"<Track#>",track)
  GetTooltip = Replace(GetTooltip,"<Artist>",artist)
  GetTooltip = Replace(GetTooltip,"<Title>",title)
  GetTooltip = Replace(GetTooltip,"<Album>",album)
  GetTooltip = Replace(GetTooltip,"<Album Artist>",albumartist)
  GetTooltip = Replace(GetTooltip,"<AlbumArtist>",albumartist)
  GetTooltip = Replace(GetTooltip,"<Genre>",genre)
  GetTooltip = Replace(GetTooltip,"<Year>",theyear)
  GetTooltip = Replace(GetTooltip,"<Bitrate>",bitrate)
  GetTooltip = Replace(GetTooltip,"<BPM>",bpm)
  GetTooltip = Replace(GetTooltip,"<Composer>",composer)
  GetTooltip = Replace(GetTooltip,"<Custom 1>",cust1)
  GetTooltip = Replace(GetTooltip,"<Custom 2>",cust2)
  GetTooltip = Replace(GetTooltip,"<Custom 3>",cust3)
  GetTooltip = Replace(GetTooltip,"<Filename>",path)
  GetTooltip = Replace(GetTooltip,"<Length>",length)
  GetTooltip = Replace(GetTooltip,"<Rating>",rating)
  GetTooltip = Replace(GetTooltip,"<Band>",band)
  GetTooltip = Replace(GetTooltip,"<Channels>",chan)
  GetTooltip = Replace(GetTooltip,"<Conductor>",cond)
  GetTooltip = Replace(GetTooltip,"<Copyright>",copy)
  GetTooltip = Replace(GetTooltip,"<Lyricist>",lyri)
  GetTooltip = Replace(GetTooltip,"<Mood>",mood)
  GetTooltip = Replace(GetTooltip,"<MusicComposer>",mcomp)
  GetTooltip = Replace(GetTooltip,"<Occasion>",occa)
  GetTooltip = Replace(GetTooltip,"<Publisher>",publ)
  GetTooltip = Replace(GetTooltip,"<Quality>",qual)
  GetTooltip = Replace(GetTooltip,"<Tempo>",tempo)
  GetTooltip = Replace(GetTooltip,"<Encoder>",enco)
  GetTooltip = Replace(GetTooltip,"<Custom 4>",cust4)
  GetTooltip = Replace(GetTooltip,"<Custom 5>",cust5)
  GetTooltip = Replace(GetTooltip,"<Disc#>",disc)
  GetTooltip = Replace(GetTooltip,"<Grouping>",group)
  GetTooltip = Replace(GetTooltip,"<Comment>",comment)
  GetTooltip = Replace(GetTooltip,"<Date>",thedate)
  GetTooltip = Replace(GetTooltip,"<OrigYear>",origyear)
  GetTooltip = Replace(GetTooltip,"<Lyrics>",lyrics)  
  GetTooltip = Replace(GetTooltip,"<StartTime>",starttime)
  GetTooltip = Replace(GetTooltip,"<StopTime>",stoptime)
  GetTooltip = Replace(GetTooltip,"<SkipCount>",skipcount)
  GetTooltip = Replace(GetTooltip,"<TrackType>",tracktype)
  GetTooltip = Replace(GetTooltip,"<Series>",series)
  GetTooltip = Replace(GetTooltip,"<Director>",director)
  GetTooltip = Replace(GetTooltip,"<Producer>",producer)
  GetTooltip = Replace(GetTooltip,"<Actors>",actors)
  GetTooltip = Replace(GetTooltip,"<ParentalRating>",parentalrating)
  GetTooltip = Replace(GetTooltip,"<EpisodeNumber>",episode)
  GetTooltip = Replace(GetTooltip,"<SeasonNumber>",season)
  GetTooltip = Replace(GetTooltip,"<Custom1>",cust1)
  GetTooltip = Replace(GetTooltip,"<Custom2>",cust2)
  GetTooltip = Replace(GetTooltip,"<Custom3>",cust3)
  GetTooltip = Replace(GetTooltip,"<Custom4>",cust4)
  GetTooltip = Replace(GetTooltip,"<Custom5>",cust5)
  GetTooltip = Replace(GetTooltip,"""","'")  
End Function

Function GetLink(url)
  GetLink = Escape(Replace(url,"\","/"))
  If TrackLinks < 2 Then
    GetLink = "file:///"&GetLink
  End If
End Function

Function GetBombs(rating)
  GetBombs = "<span style=""white-space:nowrap;"">"
  If InStr(rating,".") Then
    rating = Left(rating,InStr(rating,".")-1)
  End if   
  Dim temp : temp = Int(rating)
  Dim count : count = 0
  If temp >= 0 and temp < 6 Then 
    GetBombs = "<img src="""&ImageBase&"bomb.png"" alt=""Image"" border=""0""><img src="""&ImageBase&"bomb.png"" alt=""Image"" border=""0""><img src="""&ImageBase&"bomb.png"" alt=""Image"" border=""0""><img src="""&ImageBase&"bomb.png"" alt=""Image"" border=""0""><img src="""&ImageBase&"bomb.png"" alt=""Image"" border=""0"">"
    Exit Function
  End If
  Do While temp > 15
    GetBombs = GetBombs&"<img src="""&ImageBase&"star.png"" alt=""Image"" border=""0"">"
    count = count + 1
    temp = temp - 20
  Loop
  If temp > 5 Then 
    GetBombs = GetBombs&"<img src="""&ImageBase&"half-star2.png"" alt=""Image"" border=""0"">"  
    count = count + 1
  End If
  Do While count < 5
    GetBombs = GetBombs&"<img src="""&ImageBase&"no-star2.png"" alt=""Image"" border=""0"">"
    count = count + 1
  Loop
  GetBombs = GetBombs&"</span>"
End Function

Function GetStars(rating)
  GetStars = "<span style=""white-space:nowrap;"">"
  If InStr(rating,".") Then
    rating = Left(rating,InStr(rating,".")-1)
  End if   
  Dim temp : temp = Int(rating)
  Dim count : count = 0
  If temp < 0 Then 
    GetStars = "<img src="""&ImageBase&"no-star.png"" alt=""Image"" border=""0""><img src="""&ImageBase&"no-star.png"" alt=""Image"" border=""0""><img src="""&ImageBase&"no-star.png"" alt=""Image"" border=""0""><img src="""&ImageBase&"no-star.png"" alt=""Image"" border=""0""><img src="""&ImageBase&"no-star.png"" alt=""Image"" border=""0"">"
    Exit Function
  End If
  Do While temp > 15
    GetStars = GetStars&"<img src="""&ImageBase&"star.png"" alt=""Image"" border=""0"">"
    count = count + 1
    temp = temp - 20
  Loop
  If temp > 5 Then 
    GetStars = GetStars&"<img src="""&ImageBase&"half-star2.png"" alt=""Image"" border=""0"">"  
    count = count + 1
  End If
  Do While count < 5
    GetStars = GetStars&"<img src="""&ImageBase&"no-star2.png"" alt=""Image"" border=""0"">"
    count = count + 1
  Loop
  GetStars = GetStars&"</span>"
End Function

Function GetAlbum(id,mode,ext,cmp)
  GetAlbum = "<a id=""album_0"">"&MapXML(UnknownAlbum)&"</a>"
  If id = 0 Then 
    Exit Function
  End If
  
  Dim name,artist,length,tracks,rating,stars,theyear,bombs,iter,album,href,temp,discs
  Dim sql : sql = "AND (Songs.IDAlbum="&id&") "&FilterSQL   
  If Debug Then
    Call out("#"&sql)
  End If      
  Set iter = SDB.Database.QuerySongs(sql)
  If iter.EOF Then 
    Exit Function
  End If
  
  Set album = iter.Item.Album  
  If mode = 1 Then
    name = "<a id=""album_"&id&""">"&MapXML(album.Name)&"</a>"
    sql = "SELECT IDArtist,Artist FROM ArtistsAlbums,Artists WHERE ArtistsAlbums.IDArtist=Artists.ID AND IDAlbum="&id
    If Debug Then
      Call out("#"&sql)
    End If          
    Dim dit : Set dit = SDB.Database.OpenSQL(sql)
    While Not (dit.EOF)
      href = "#artist_"&dit.ValueByIndex(0)
      If SeparateFiles Then 
        href = "../artists/artist_"&dit.ValueByIndex(0)&ext
      End If
      If FramedView Then 
        href = "../artists/artist_"&dit.ValueByIndex(0)&ext&""" target=""main"
      End If
      artist = artist&Delimiter&"<a href="""&href&""">"&MapXML(dit.StringByIndex(1))&"</a>"        
      dit.Next
    WEnd
    If Len(artist) > 0 Then
      artist = Mid(artist,3)
    End If
  Else
    name = MapXML(album.Name)
    artist = MapXML(album.Artist.Name)
  End If

  href = "SELECT Avg(Rating) As AvgRating,CAST(Avg(Year/10000) AS INTEGER) As AvgYear,Max(DiscNumber) As MaxDisc FROM Songs WHERE IDAlbum="&id&" "&FilterSQL
  If Debug Then
    Call out("#"&href)
  End If            
  Set iter = SDB.Database.OpenSQL(href) 
  length = GetTime(album.AlbumLength/1000)
  tracks = album.Tracks.Count  
  temp = FixNull(iter.ValueByName("AvgRating"),-1)
  stars = GetStars(temp)
  bombs = GetBombs(temp)
  If temp < 0 Then 
    rating = "?"
  Else
    temp = temp / 20
    If temp = Int(temp) Then
      rating = temp
    Else
      rating = FormatNumber(temp,1)
    End If
  End If
  temp = Int(FixNull(iter.ValueByName("AvgYear"),0))
  If temp < 0 Then
    theyear = "????"
  Else
    theyear = Left(CStr(temp),4)
  End If 
  discs = iter.StringByName("MaxDisc")
  If discs = "" Then
    discs = "1"
  End If

  If mode = 2 Then
    GetAlbum = DetailsMask
  Else
    If (cmp = "Y") And Not (CompAlbumMask = "") Then
      GetAlbum = CompAlbumMask
    Else
      GetAlbum = AlbumMask
    End If 
  End If
  GetAlbum = Replace(GetAlbum,"<Name>",name)
  GetAlbum = Replace(GetAlbum,"<Artist>",artist)
  GetAlbum = Replace(GetAlbum,"<Length>",length)
  GetAlbum = Replace(GetAlbum,"<Tracks>",tracks)
  GetAlbum = Replace(GetAlbum,"<Rating>",rating)
  GetAlbum = Replace(GetAlbum,"<Year>",theyear)  
  GetAlbum = Replace(GetAlbum,"<Stars>",stars)
  GetAlbum = Replace(GetAlbum,"<Bombs>",bombs)
  GetAlbum = Replace(GetAlbum,"<Discs>",discs)
  If InStr(GetAlbum,"<Comment>") > 0 Then
    Dim comment : comment = ""
    href = "SELECT Comment FROM Albums WHERE ID="&id
    If Debug Then
      Call out("#"&href)
    End If            
    Set iter = SDB.Database.OpenSQL(href)
    If Not iter.EOF Then
      comment = iter.StringByIndex(0)
    End If     
    GetAlbum = Replace(GetAlbum,"<Comment>",comment)
  End If
  If InStr(GetAlbum,"<Genre>") > 0 Then
    Dim genre : genre = ""
    href = "SELECT DISTINCT Genres.GenreName FROM Genres,Songs,GenresSongs WHERE GenresSongs.IDSong=Songs.ID AND GenresSongs.IDGenre=Genres.IDGenre AND Songs.IDAlbum="&id&" "&FilterSQL&"ORDER BY Genres.GenreName"
    If Debug Then
      Call out("#"&href)
    End If            
    Set iter = SDB.Database.OpenSQL(href)
    If Not iter.EOF Then
      genre = iter.StringByName("GenreName")
      iter.Next
      While Not iter.EOF
        genre = genre&", "&iter.StringByName("GenreName")
        iter.Next
      WEnd
    End If
    GetAlbum = Replace(GetAlbum,"<Genre>",genre)
  End If
  
  If mode = 1 Then
    GetAlbum = "<a id=""album_"&id&""">"&GetAlbum&"</a>"
  End If  
End Function

Function GetArtist(id,name)
  GetArtist = "<a id=""artist_0"">"&MapXML(UnknownArtist)&"</a>"
  If id = 0 Then 
    Exit Function
  End If
  GetArtist = "<a id=""artist_"&id&""">"&MapXML(name)&"</a>"
  If ArtistMask = "" Then
    Exit Function
  End If
  
  Dim sql : sql = "SELECT Comment,Tracks,Albums,Authors,Conducts,Lyrics FROM Artists WHERE ID="&id
  If Debug Then
    Call out("#"&sql)
  End If            
  Dim iter : Set iter = SDB.Database.OpenSQL(sql)
  If iter.EOF Then
    Exit Function
  End If
  Dim comment : comment = iter.StringByIndex(0)
  Dim tracks : tracks = FixNull(iter.ValueByIndex(1),0)
  Dim albums : albums = FixNull(iter.ValueByIndex(2),0)
  Dim authors : authors = FixNull(iter.ValueByIndex(3),0)
  Dim conducts : conducts = FixNull(iter.ValueByIndex(4),0)
  Dim lyrics : lyrics = FixNull(iter.ValueByIndex(5),0)  
  
  sql = "SELECT Avg(Rating) As AvgRating,CAST(Avg(Year/10000) AS INTEGER) As AvgYear FROM Songs,ArtistsSongs WHERE Songs.ID=ArtistsSongs.IDSong AND ArtistsSongs.IDArtist="&id&" AND ArtistsSongs.PersonType=1 "&FilterSQL
  If Debug Then
    Call out("#"&sql)
  End If            
  Set iter = SDB.Database.OpenSQL(sql) 
  Dim temp : temp = FixNull(iter.ValueByName("AvgRating"),-1)  
  Dim stars : stars = GetStars(temp)
  Dim bombs : bombs = GetBombs(temp)
  Dim rating : rating = "?"
  If temp > -1 Then 
    temp = temp / 20
    If temp = Int(temp) Then
      rating = temp
    Else
      rating = FormatNumber(temp,1)
    End If
  End If
  Dim theyear : theyear = "????"  
  temp = Int(FixNull(iter.ValueByName("AvgYear"),0))
  If temp > -1 Then
    theyear = Left(CStr(temp),4)
  End If 

  GetArtist = ArtistMask 
  GetArtist = Replace(GetArtist,"<Name>",MapXML(name))
  GetArtist = Replace(GetArtist,"<Tracks>",tracks)
  GetArtist = Replace(GetArtist,"<Albums>",albums)
  GetArtist = Replace(GetArtist,"<Authors>",authors)
  GetArtist = Replace(GetArtist,"<Conducts>",Conducts)
  GetArtist = Replace(GetArtist,"<Lyrics>",lyrics)
  GetArtist = Replace(GetArtist,"<Rating>",rating)
  GetArtist = Replace(GetArtist,"<Year>",theyear)  
  GetArtist = Replace(GetArtist,"<Stars>",stars)
  GetArtist = Replace(GetArtist,"<Bombs>",bombs)
  GetArtist = Replace(GetArtist,"<Comment>",MapXML(comment))
  
  GetArtist = "<a id=""artist_"&id&""">"&GetArtist&"</a>"
End Function

Function GetUnknown(id,ext,sql)
  GetUnknown = "<a id=""album_0"">"&MapXML(UnknownAlbum)&"</a>"
  Dim href : href = "SELECT Artist FROM Artists WHERE ID="&id
  If Debug Then
    Call out("#"&href)
  End If            
  Dim iter : Set iter = SDB.Database.OpenSQL(href)
  If iter.EOF Then Exit Function
  
  Dim name : name = "<a id=""album_"&id&"u"">"&MapXML(UnknownAlbum)&"</a>"
  href = "#artist_"&id
  If SeparateFiles Then 
    href = "../artists/artist_"&id&ext
  End If
  If FramedView Then 
    href = "../artists/artist_"&id&ext&""" target=""main"
  End If
  Dim artist : artist = "<a href="""&href&""">"&MapXML(iter.StringByIndex(0))&"</a>"
  Set iter = Nothing
  
  href = "SELECT Sum(SongLength),Count(*),Avg(Rating),CAST(Avg(Year/10000) AS INTEGER),Max(DiscNumber)"&Mid(sql,15)
  If Debug Then
    Call out("#"&href)
  End If           
  Set iter = SDB.Database.OpenSQL(href) 
  Dim length : length = GetTime(iter.ValueByIndex(0)/1000)
  Dim tracks : tracks = FixNull(iter.ValueByIndex(1),0)
  Dim temp : temp = FixNull(iter.ValueByIndex(2),-1) 
  Dim stars : stars = GetStars(temp)
  Dim bombs : bombs = GetBombs(temp)
  Dim rating : rating = "?"
  If temp >= 0 Then 
    temp = temp / 20
    If temp = Int(temp) Then
      rating = temp
    Else
      rating = FormatNumber(temp,1)
    End If
  End If
  Dim theyear : theyear = "????"
  temp = iter.ValueByIndex(3)
  If temp > 0 Then
    theyear = Left(CStr(temp),4)
  End If
  Dim discs : discs = iter.StringByIndex(4)
  If discs = "" Then
    discs = "1"
  End If

  GetUnknown = AlbumMask
  GetUnknown = Replace(GetUnknown,"<Name>",name)
  GetUnknown = Replace(GetUnknown,"<Artist>",artist)
  GetUnknown = Replace(GetUnknown,"<Length>",length)
  GetUnknown = Replace(GetUnknown,"<Tracks>",tracks)
  GetUnknown = Replace(GetUnknown,"<Rating>",rating)
  GetUnknown = Replace(GetUnknown,"<Year>",theyear)  
  GetUnknown = Replace(GetUnknown,"<Stars>",stars)
  GetUnknown = Replace(GetUnknown,"<Bombs>",bombs)
  GetUnknown = Replace(GetUnknown,"<Discs>",discs)
  GetUnknown = Replace(GetUnknown,"<Comment>","")
  
  GetUnknown = "<a id=""album_"&id&"u"">"&GetUnknown&"</a>"
End Function

Function GetAlbumArt(id)
  GetAlbumArt = Artwork.Item(id)
  If Not GetAlbumArt = "" Then Exit Function
  
  If NewImages Then
    If SDB.Tools.FileSystem.FileExists(ImgsDir&id&".jpg") Then
      GetAlbumArt = ImageBase&id&".jpg"
      Exit Function
    End If
  End If
  
  GetAlbumArt = ImageBase&"none.jpg"
  If id = 0 Then
    Artwork.Item(id) = GetAlbumArt
    Exit Function
  End If  
  
  Dim sql : sql = "AND (Songs.IDAlbum="&id&") "&FilterSQL 
  If Debug Then
    Call out("#"&sql)
  End If     
  Dim iter : Set iter = SDB.Database.QuerySongs(sql)
  If iter.EOF Then 
    Artwork.Item(id) = GetAlbumArt
    Exit Function
  End If
  
  Dim pics : Set pics = iter.Item.Album.AlbumArt
  If pics Is Nothing Then 
    Artwork.Item(id) = GetAlbumArt
    Exit Function
  End If
  
  Dim i,img,h,w,jpg,img2
  For i = 0 To pics.Count-1 
    Set img = pics.Item(i).Image
    If Not (img Is Nothing) Then
      w = AlbumArtWidth
      h = AlbumArtWidth
      Select Case AlbumArtMode
        Case 0 
          If img.Width>0 Then
            h = w * img.Height / img.Width
          Else
            h = 0
          End If
        Case 1
          If img.Height>0 Then
            w = h * img.Width / img.Height
          Else
            w = 0
          End If
      End Select
      Set jpg = img.ConvertFormat(w,h,"image/jpeg",85,0)

      If Not (jpg Is Nothing) Then
        GetAlbumArt = ImgsDir&id&".jpg"
        Set img2 = SDB.Tools.FileSystem.CreateTextFile(GetAlbumArt,True)
        img2.WriteData jpg.ImageData, jpg.ImageDataLen
        img2.Close
        Set img2 = Nothing
        GetAlbumArt = ImageBase&id&".jpg"
        Exit For
      End If
    End If
  Next
  
  Artwork.Item(id) = GetAlbumArt
End Function

Function UpdateProgress(txt)
  UpdateProgress = AppName&" - "&txt
  Progress.Text = UpdateProgress
  SDB.ProcessMessages
  If Debug Then
    Call out(txt)
  End If
End Function

Function GetTime(sec)  
  Dim min : min = 0
  sec = Int(sec)
  If sec < 0 Then
    sec = sec+86400
  End If  
  Do While sec > 59 
    sec = sec - 60
    min = min + 1
  Loop
  If sec < 10 Then
    GetTime = min&":0"&sec
  Else
    GetTime = min&":"&sec
  End If
End Function

Function GeneratePath(pFolderPath)
  GeneratePath = False
  If Not fso.FolderExists(pFolderPath) Then
    If GeneratePath(fso.GetParentFolderName(pFolderPath)) Then
      GeneratePath = True
      Call fso.CreateFolder(pFolderPath)
    End If
  Else
    GeneratePath = True
  End If
End Function

Function GetFilterID(Name)
  Dim sql : sql = "SELECT COUNT(*) FROM Filters WHERE Name='"&Replace(Name,"'","''")&"'"
  If Debug Then
    Call out("#"&sql)
  End If       
  If SDB.Database.OpenSQL(sql).ValueByIndex(0) = 0 Then
    Dim s : s = "The filter '"&Name&"' no longer exists. Do you wish to continue without a filter?"
    If SDB.MessageBox(s,mtWarning,Array(mbYes,mbNo)) = mrYes Then
      FilterName = "- No Filter -"
      SDB.IniFile.StringValue("AdvancedReport","FilterName") = FilterName
      GetFilterID = -1
    Else
      GetFilterID = -2
    End If
  Else
    sql = "SELECT ID FROM Filters WHERE Name='"&Replace(Name,"'","''")&"'"
    If Debug Then
      Call out("#"&sql)
    End If  
    GetFilterID = SDB.Database.OpenSQL(sql).ValueByIndex(0)
  End If
End Function

Sub FillDropDownFromArray(DropDown,SourceArray)
  Dim i : i = 0
  For i = 0 To UBound(SourceArray)
    Call DropDown.AddItem(SourceArray(i))
  Next
End Sub

Sub CreateFiltersArray
  Dim sql : sql = "SELECT COUNT(*) FROM Filters" 
  If Debug Then
    Call out("#"&sql)
  End If  
  Redim Filters(SDB.Database.OpenSQL(sql).ValueByIndex(0)+2)
  Filters(0) = "- No Filter -"
  Filters(1) = "- Active Filter -"
  Filters(2) = "- Selected Tracks -"
  Dim i : i = 3
  sql = "SELECT Name FROM Filters ORDER BY Pos"
  If Debug Then
    Call out("#"&sql)
  End If    
  Dim iter : Set iter = SDB.Database.OpenSQL(sql)
  Do While Not iter.EOF
    Filters(i) = iter.StringByIndex(0)
    i = i+1
    iter.Next
  Loop
  Set iter = Nothing
End Sub

Function GetFiltersArrayID(Name)
  Dim i : i = 0
  For i = 0 To UBound(Filters)
    If Filters(i) = Name Then
      GetFiltersArrayID = i
      Exit Function
    End If
  Next
  GetFiltersArrayID = 0
End Function

Function GetFilterSQL(Name)
  GetFilterSQL = ""
  Dim sql : sql = ""
  Select Case Name
    Case "- No Filter -"
      GetFilterSQL = ""
    Case "- Active Filter -"
      sql = SDB.Database.ActiveFilterQuery
      If Not (sql = "") Then 
        GetFilterSQL = "AND "&sql
      End If      
    Case "- Selected Tracks -"
      GetFilterSQL = "AND Songs.ID IN (SELECT tmpIDSong FROM tmpSelected)"
    Case Else
      Dim id : id = GetFilterID(Name)
      If id > -1 Then
        sql = SDB.Database.GetFilterQuery(id)
        If Not (sql = "") Then 
          GetFilterSQL = "AND "&sql
        End If
      Else
        If id = -2 Then
          GetFilterSQL = "~EXIT~"
        End If
      End If
   End Select
End Function

Sub SortArray(arr)
  If UBound(arr) > 0 Then
    Call QuickSort(arr,LBound(arr),UBound(arr))
  End If
End Sub

Sub QuickSort(vec,loBound,hiBound)
  Dim pivot,loSwap,hiSwap,temp

  If hiBound - loBound = 1 Then
    If Vec(loBound) > Vec(hiBound) Then
      temp = Vec(loBound)
      Vec(loBound) = Vec(hiBound)
      Vec(hiBound) = temp
    End If
  End If

  pivot = Vec(Int((loBound + hiBound) / 2))
  Vec(Int((loBound + hiBound) / 2)) = Vec(loBound)
  Vec(loBound) = pivot
  loSwap = loBound + 1
  hiSwap = hiBound
  
  Do
    While loSwap < hiSwap And Vec(loSwap) <= pivot
      loSwap = loSwap + 1
    WEnd
    While Vec(hiSwap) > pivot
      hiSwap = hiSwap - 1
    WEnd
    If loSwap < hiSwap then
      temp = Vec(loSwap)
      Vec(loSwap) = Vec(hiSwap)
      Vec(hiSwap) = temp
    End If
  Loop While loSwap < hiSwap
  
  Vec(loBound) = Vec(hiSwap)
  Vec(hiSwap) = pivot
  
  If loBound < (hiSwap - 1) Then 
    Call QuickSort(vec,loBound,hiSwap-1)
  End If
  If hiSwap + 1 < hibound Then 
    Call QuickSort(vec,hiSwap+1,hiBound)
  End If
End Sub 

Sub GetPlaylist(num,nam,ply,lev)
  Set ply = SDB.PlaylistByTitle(nam)
  If Not (ply Is Nothing) Then
    If num = 0 Then
      nam = ply.Title
      Exit Sub
    Else
      lev = lev+1
      Dim l : l = lev
      Dim i : i = 0
      Dim kids : Set kids = ply.ChildPlaylists
      For i = 0 To kids.Count-1
        num = num-1
        nam = kids.Item(i).Title
        Call GetPlaylist(num,nam,ply,lev)
        If num = 0 Then
          Exit For
        End If
        lev = l
      Next
    End If
  End If
End Sub

Function GetIndent(lev)
  Dim i : i = 0
  GetIndent = "- "
  For i = 1 To lev
    GetIndent = "      &nbsp;&nbsp;&nbsp;"&GetIndent
  Next
End Function

Sub Install()
  Dim inip : inip = SDB.ScriptsPath&"Scripts.ini"
  Dim inif : Set inif = SDB.Tools.IniFileByPath(inip)
  If Not (inif Is Nothing) Then
    inif.StringValue("AdvancedReport","Filename") = "AdvancedReport.vbs"
    inif.StringValue("AdvancedReport","Procname") = "AdvancedReport"
    inif.StringValue("AdvancedReport","Order") = "22"
    inif.StringValue("AdvancedReport","DisplayName") = "Advanced Report"
    inif.StringValue("AdvancedReport","Description") = "Create an advanced html report"
    inif.StringValue("AdvancedReport","Language") = "VBScript"
    inif.StringValue("AdvancedReport","ScriptType") = "1"
    SDB.RefreshScriptItems
  End If
End Sub

Function GetRelative(relp,path)
  GetRelative = relp
  Dim temp : temp = Left(path,InStrRev(path,"\"))
 
  'check same folder
  If InStr(relp,temp) = 1 Then
    GetRelative = Mid(relp,Len(temp)+1)
    Exit Function
  End If
 
  'check parent folders
  If Left(temp,2) = "\\" Then
    temp = Mid(temp,3)
  End If
  If Left(relp,2) = "\\" Then
    relp = Mid(relp,3)
  End If
  Dim tp : tp = Left(temp,InStr(temp,"\"))
  Dim tr : tr = Left(relp,InStr(relp,"\"))
  If (UCase(tp) = UCase(tr)) Then
    While (UCase(tp) = UCase(tr))
      temp = Mid(temp,Len(tp)+1)
      relp = Mid(relp,Len(tr)+1)
      tp = Left(temp,InStr(temp,"\"))
      tr = Left(relp,InStr(relp,"\"))
    WEnd
    GetRelative = ""
    While (InStr(temp,"\") > 0)
      GetRelative = GetRelative&"..\"
      temp = Mid(temp,Len(tp)+1)
      tp = Left(temp,InStr(temp,"\"))
    WEnd
    GetRelative = GetRelative&relp
  End If
End Function

Function FixYears(str)
  If Left(str,2) = "-1" Then
    str = Mid(str,3)
    If Left(str,2) = ", " Then
      str = Mid(str,3)
    End If
  End If
  If (str = "") Or (InStr(str,", ") = 0) Then
    FixYears = str
  Else
    Dim arr : arr = Split(str,", ")
    Dim i : i = 0
    Dim j : j = Int(arr(0))
    Dim b : b = False
    FixYears = arr(0)
    For i = 1 To UBound(arr)
      If Int(arr(i)) = j+1 Then
        b = True
      Else
        If b Then
          b = False
          FixYears = FixYears&"-"&j
        End If
        FixYears = FixYears&", "&arr(i)
      End If
      j = Int(arr(i))
    Next
    If b Then
      FixYears = FixYears&"-"&j
    End If
  End If
End Function

Function BuildHeader(mask)
  BuildHeader = mask
  Dim i : i = 0
  If InStr(BuildHeader,"<Custom ") > 0 Then
    For i = 1 To 5 
      If InStr(BuildHeader,"<Custom "&i&">") > 0 Then
        BuildHeader = Replace(BuildHeader,"<Custom "&i&">",SDB.IniFile.StringValue("CustomFields","Fld"&i&"Name"))
      End If
    Next
  End If
  If InStr(BuildHeader,"<Custom") > 0 Then
    For i = 1 To 5 
      If InStr(BuildHeader,"<Custom"&i&">") > 0 Then
        BuildHeader = Replace(BuildHeader,"<Custom"&i&">",SDB.IniFile.StringValue("CustomFields","Fld"&i&"Name"))
      End If
    Next
  End If  
  BuildHeader = Replace(BuildHeader,"<Track#>",SDB.Localize("Track"))
  BuildHeader = Replace(BuildHeader,"<Artist>",SDB.Localize("Artist"))
  BuildHeader = Replace(BuildHeader,"<Title>",SDB.Localize("Title"))
  BuildHeader = Replace(BuildHeader,"<Album>",SDB.Localize("Album"))
  BuildHeader = Replace(BuildHeader,"<Album Artist>",SDB.Localize("Album Artist"))
  BuildHeader = Replace(BuildHeader,"<AlbumArtist>",SDB.Localize("Album Artist"))
  BuildHeader = Replace(BuildHeader,"<Genre>",SDB.Localize("Genre"))
  BuildHeader = Replace(BuildHeader,"<Year>",SDB.Localize("Year"))
  BuildHeader = Replace(BuildHeader,"<Bitrate>",SDB.Localize("Bitrate"))
  BuildHeader = Replace(BuildHeader,"<BPM>",SDB.Localize("BPM"))
  BuildHeader = Replace(BuildHeader,"<Composer>",SDB.Localize("Composer"))
  BuildHeader = Replace(BuildHeader,"<Filename>",SDB.Localize("Filename"))
  BuildHeader = Replace(BuildHeader,"<Length>",SDB.Localize("Length"))
  BuildHeader = Replace(BuildHeader,"<Rating>",SDB.Localize("Rating"))
  BuildHeader = Replace(BuildHeader,"<Stars>",SDB.Localize("Rating"))
  BuildHeader = Replace(BuildHeader,"<Bombs>",SDB.Localize("Rating"))
  BuildHeader = Replace(BuildHeader,"<Band>",SDB.Localize("Band"))
  BuildHeader = Replace(BuildHeader,"<Channels>",SDB.Localize("Channels"))
  BuildHeader = Replace(BuildHeader,"<MusicComposer>",SDB.Localize("Author"))
  BuildHeader = Replace(BuildHeader,"<Conductor>",SDB.Localize("Conductor"))
  BuildHeader = Replace(BuildHeader,"<Copyright>",SDB.Localize("Copyright"))
  BuildHeader = Replace(BuildHeader,"<Disc#>",SDB.Localize("Disc"))
  BuildHeader = Replace(BuildHeader,"<Grouping>",SDB.Localize("Grouping"))
  BuildHeader = Replace(BuildHeader,"<Lyricist>",SDB.Localize("Lyricist"))
  BuildHeader = Replace(BuildHeader,"<Mood>",SDB.Localize("Mood"))
  BuildHeader = Replace(BuildHeader,"<Occasion>",SDB.Localize("Occasion"))
  BuildHeader = Replace(BuildHeader,"<Publisher>",SDB.Localize("Publisher"))
  BuildHeader = Replace(BuildHeader,"<Quality>",SDB.Localize("Quality"))
  BuildHeader = Replace(BuildHeader,"<Tempo>",SDB.Localize("Tempo"))
  BuildHeader = Replace(BuildHeader,"<Encoder>",SDB.Localize("Encoder"))
  BuildHeader = Replace(BuildHeader,"<Link>",SDB.Localize("Link"))
  BuildHeader = Replace(BuildHeader,"<ArtistLink>",SDB.Localize("Artist"))
  BuildHeader = Replace(BuildHeader,"<TitleLink>",SDB.Localize("Title"))
  BuildHeader = Replace(BuildHeader,"<Comment>",SDB.Localize("Comment"))
  BuildHeader = Replace(BuildHeader,"<MediaIcon>",SDB.Localize("Media"))
  BuildHeader = Replace(BuildHeader,"<Date>",SDB.Localize("Date"))
  BuildHeader = Replace(BuildHeader,"<OrigYear>",SDB.Localize("Original Year"))
  BuildHeader = Replace(BuildHeader,"<Lyrics>",SDB.Localize("Lyrics"))
  BuildHeader = Replace(BuildHeader,"<StartTime>",SDB.Localize("Start Time"))
  BuildHeader = Replace(BuildHeader,"<StopTime>",SDB.Localize("Stop Time"))
  BuildHeader = Replace(BuildHeader,"<SkipCount>",SDB.Localize("Skip Count"))
  BuildHeader = Replace(BuildHeader,"<TrackType>",SDB.Localize("Track Type"))
  BuildHeader = Replace(BuildHeader,"<Series>",SDB.Localize("Series"))
  BuildHeader = Replace(BuildHeader,"<Director>",SDB.Localize("Director"))
  BuildHeader = Replace(BuildHeader,"<Producer>",SDB.Localize("Producer"))
  BuildHeader = Replace(BuildHeader,"<Actors>",SDB.Localize("Actors"))
  BuildHeader = Replace(BuildHeader,"<ParentalRating>",SDB.Localize("Parental Rating"))
  BuildHeader = Replace(BuildHeader,"<EpisodeNumber>",SDB.Localize("Episode"))
  BuildHeader = Replace(BuildHeader,"<SeasonNumber>",SDB.Localize("Season"))
  BuildHeader = Replace(BuildHeader,"<|>","</th><th>")
End Function    

Sub clear()
  Dim wsh : Set wsh = CreateObject("WScript.Shell")
  Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
  Dim loc : loc = wsh.ExpandEnvironmentStrings("%TEMP%")
  If Right(loc,1) = "\" Then
    loc = loc&"AdvancedReport.log"
  Else
    loc = loc&"\AdvancedReport.log"
  End If
  Dim logf : Set logf = fso.CreateTextFile(loc,True)
  Call logf.Close()
End Sub          

Sub out(txt)
  Dim wsh : Set wsh = CreateObject("WScript.Shell")
  Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
  Dim loc : loc = wsh.ExpandEnvironmentStrings("%TEMP%")
  If Right(loc,1) = "\" Then
    loc = loc&"AdvancedReport.log"
  Else
    loc = loc&"\AdvancedReport.log"
  End If
  Dim logf : Set logf = fso.OpenTextFile(loc,8,True)
  Call logf.WriteLine(Time&Chr(9)&SDB.ToAscii(txt))
  Call logf.Close()
End Sub

Function FixNull(num,alt)
  If (IsNull(num)) Or (num = "") Then
    FixNull = alt
  Else
    FixNull = num
  End If
End Function

Function BuildLinks(mode,dic,id,name)
  BuildLinks = ""
  Dim i : i = dic.Count
  If i < 1 Then
    Exit Function
  End If
  Dim alb : alb = ""
  Dim art : art = ""
  Select Case mode
    Case 0 'artist
      art = name
    Case 1 'album
      alb = name
      Dim sql : sql = "SELECT Album,Artist FROM Albums WHERE ID="&id
      If Debug Then
        Call out("#"&sql)
      End If
      Dim iter : Set iter = SDB.Database.OpenSQL(sql)      
      If Not (iter.EOF) Then
        alb = iter.StringByIndex(0)
        art = iter.StringByIndex(1)
      End IF
  End Select
  Dim trg : trg = "_blank"
  If FramedView Then 
    trg = "main"
  End If   
  Dim arr : arr = dic.Keys
  For i = 0 To UBound(arr)
    Dim cap : cap = MapXML(arr(i))
    Dim url : url = dic.Item(arr(i))
    url = Replace(url,"%album%",alb)
    url = Replace(url,"%albumartist%",art)
    url = Replace(url,"%artist%",art)
    BuildLinks = BuildLinks&"<a href="""&url&""" target="""&trg&""">"&cap&"</a> "
  Next
End Function